@@ -2,14 +2,14 @@
 	"translatorID": "2dc0b23d-64d8-4933-b629-5c003451ccf7",
 	"label": "Légifrance",
 	"creator": "Guillaume Adreani",
-	"target": "^https?://(www.)?legifrance\\.gouv\\.fr/",
+	"target": "^https?://(www\\.)?legifrance\\.gouv\\.fr/",
 	"minVersion": "2.1.9",
 	"maxVersion": "",
 	"priority": 100,
 	"inRepository": true,
 	"translatorType": 4,
 	"browserSupport": "gcsibv",
-	"lastUpdated": "2013-04-17 03:09:28"
+	"lastUpdated": "2021-06-07 16:46:17"
 }
 
 /*
@@ -59,7 +59,7 @@ var legifrancecaseRegexp = /https?:\/\/(www.)?legifrance\\.gouv\\.fr\/.+JURITEXT
 
 		// Paramètres communs
 
-		var title = ZU.xpathText(doc, '//h2[@class="title"]');
+		var title = ZU.xpathText(doc, '//h1[@class="main-title"]');
 		newItem.title = title;
 		newItem.url = doc.location.href;
 		var rtfurl = ZU.xpathText(doc, '//a[contains(text(), "Télécharger")]/@href');
@@ -215,7 +215,7 @@ var legifrancecaseRegexp = /https?:\/\/(www.)?legifrance\\.gouv\\.fr\/.+JURITEXT
 
 		var newItem = new Zotero.Item("statute");
 
-		var title = ZU.xpathText(doc, '//h2[@class="title"]');
+		var title = ZU.xpathText(doc, '//h1[@class="main-title"]');
 		newItem.title = title;
 		newItem.accessDate = 'CURRENT_TIMESTAMP';
 
@@ -638,4 +638,4 @@ var testCases = [
 		]
 	}
 ]
-/** END TEST CASES **/
\ No newline at end of file
+/** END TEST CASES **/
