@@ -28,6 +28,13 @@ Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/osfile.jsm");
 Components.utils.import("resource://zotero/config.js");
 
+var React = require('react');
+var ReactDOM = require('react-dom');
+var VirtualizedTable = require('components/virtualized-table');
+var { getDOMElement } = require('components/icons');
+var { IntlProvider } = require('react-intl');
+var { renderCell } = VirtualizedTable;
+
 Zotero_Preferences.Sync = {
   checkmarkChar: '\u2705',
   noChar: '\uD83D\uDEAB',
@@ -163,7 +170,13 @@ Zotero_Preferences.Sync = {
       return;
     }
 
-    if (!(yield Zotero.Sync.Data.Local.checkUser(window, json.userID, json.username))) {
+    var ok = yield Zotero.Sync.Data.Local.checkUser(
+    window,
+    json.userID,
+    json.username,
+    json.displayName);
+
+    if (!ok) {
       // createAPIKeyFromCredentials will have created an API key,
       // but user decided not to use it, so we remove it here.
       Zotero.Sync.Runner.deleteAPIKey();
@@ -173,9 +186,9 @@ Zotero_Preferences.Sync = {
   }),
 
   /**
-       * Updates the auth indicator icon, depending on status
-       * @param {string} status
-       */
+   * Updates the auth indicator icon, depending on status
+   * @param {string} status
+   */
   updateSyncIndicator: function (status) {
     var img = document.getElementById('sync-status-indicator');
 
@@ -227,102 +240,124 @@ Zotero_Preferences.Sync = {
   },
 
 
-  dblClickLibraryToSync: function (event) {
-    var tree = document.getElementById("libraries-to-sync-tree");
-    var row = {},col = {},child = {};
-    tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, child);
-
-    // Below the list or on checkmark column
-    if (!col.value || col.value.element.id == 'libraries-to-sync-checked') {
-      return;
-    }
-    // if dblclicked anywhere but the checkbox update pref
-    return this.toggleLibraryToSync(row.value);
-  },
-
-
-  clickLibraryToSync: function (event) {
-    var tree = document.getElementById("libraries-to-sync-tree");
-    var row = {},col = {},child = {};
-    tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, child);
-
-    // Below the list or not on checkmark column
-    if (!col.value || col.value.element.id != 'libraries-to-sync-checked') {
-      return;
-    }
-    // if clicked on checkbox update pref
-    return this.toggleLibraryToSync(row.value);
-  },
-
-
   toggleLibraryToSync: function (index) {
-    var treechildren = document.getElementById('libraries-to-sync-rows');
-    if (index >= treechildren.childNodes.length) {
-      return;
-    }
-    var row = treechildren.childNodes[index];
-    var val = row.firstChild.childNodes[1].getAttribute('value');
-    if (!val) {
-      return;
+    if (typeof index != "number") {
+      index = this._tree.selection.focused;
     }
+    if (index == -1 || !this._rows[index].editable) return;
+    const row = this._rows[index];
+    this._rows[index].checked = !this._rows[index].checked;
+    this._tree.invalidateRow(index);
 
     var librariesToSkip = JSON.parse(Zotero.Prefs.get('sync.librariesToSkip') || '[]');
-    var indexOfId = librariesToSkip.indexOf(val);
+    var indexOfId = librariesToSkip.indexOf(row.id);
     if (indexOfId == -1) {
-      librariesToSkip.push(val);
-    } else {
+      librariesToSkip.push(row.id);
+    } else
+    {
       librariesToSkip.splice(indexOfId, 1);
     }
     Zotero.Prefs.set('sync.librariesToSkip', JSON.stringify(librariesToSkip));
-
-    var cell = row.firstChild.firstChild;
-    var spacing = Zotero.isWin ? '  ' : '   ';
-    cell.setAttribute('label', spacing + (indexOfId != -1 ? this.checkmarkChar : this.noChar));
-    cell.setAttribute('value', indexOfId != -1);
   },
 
 
-  initLibrariesToSync: Zotero.Promise.coroutine(function* () {
-    var tree = document.getElementById("libraries-to-sync-tree");
-    var treechildren = document.getElementById('libraries-to-sync-rows');
-    while (treechildren.hasChildNodes()) {
-      treechildren.removeChild(treechildren.firstChild);
-    }
+  initLibrariesToSync: async function () {
+    const columns = [
+    {
+      dataKey: "checked",
+      label: "zotero.preferences.sync.librariesToSync.sync",
+      fixedWidth: true,
+      // TODO: Specify in ems?
+      width: '45' },
+
+    {
+      dataKey: "name",
+      label: "zotero.preferences.sync.librariesToSync.library" }];
+
+
+    this._rows = [];
+    let renderItem = (index, selection, oldDiv = null, columns) => {
+      const row = this._rows[index];
+      let div;
+      if (oldDiv) {
+        div = oldDiv;
+        div.innerHTML = "";
+      } else
+      {
+        div = document.createElementNS("http://www.w3.org/1999/xhtml", 'div');
+        div.className = "row";
+        div.addEventListener('dblclick', () => {
+          this.toggleLibraryToSync(index);
+        });
+      }
+      div.classList.toggle('selected', selection.isSelected(index));
+
+      for (let column of columns) {
+        if (column.dataKey === 'checked') {
+          let span = document.createElementNS("http://www.w3.org/1999/xhtml", 'span');
+          span.className = `cell ${column.className}`;
+          if (row.id != 'loading') {
+            span.innerText = row.checked ? this.checkmarkChar : this.noChar;
+            span.style.textAlign = 'center';
+          }
+          span.addEventListener('mousedown', () => {
+            this.toggleLibraryToSync(index);
+          });
+          span.style.pointerEvents = 'initial';
+          div.appendChild(span);
+        } else
+        {
+          div.appendChild(renderCell(index, row[column.dataKey], column));
+        }
+      }
+      return div;
+    };
+    let handleKeyDown = (e) => {
+      if (e.key == ' ') {
+        this.toggleLibraryToSync();
+        return false;
+      }
+    };
+    let elem = /*#__PURE__*/
+    React.createElement(IntlProvider, { locale: Zotero.locale, messages: Zotero.Intl.strings }, /*#__PURE__*/
+    React.createElement(VirtualizedTable, {
+      getRowCount: () => this._rows.length,
+      id: "librariesToSync-table",
+      ref: (ref) => this._tree = ref,
+      renderItem: renderItem,
+      showHeader: true,
+      columns: columns,
+      staticColumns: true,
+      getRowString: (index) => this._rows[index].name,
+      disableFontSizeScaling: true,
+      onKeyDown: handleKeyDown }));
+
+
+
+
+    ReactDOM.render(elem, document.getElementById("libraries-to-sync-tree"));
 
     var addRow = function (libraryName, id, checked = false, editable = true) {
-      var treeitem = document.createElement('treeitem');
-      var treerow = document.createElement('treerow');
-      var checkboxCell = document.createElement('treecell');
-      var nameCell = document.createElement('treecell');
+      this._rows.push({
+        name: libraryName,
+        id,
+        checked,
+        editable });
 
-      nameCell.setAttribute('label', libraryName);
-      nameCell.setAttribute('value', id);
-      nameCell.setAttribute('editable', false);
-      var spacing = Zotero.isWin ? '  ' : '   ';
-      checkboxCell.setAttribute(
-      'label',
-      id == 'loading' ? '' : spacing + (checked ? this.checkmarkChar : this.noChar));
-
-      checkboxCell.setAttribute('value', checked);
-      checkboxCell.setAttribute('editable', false);
-
-      treerow.appendChild(checkboxCell);
-      treerow.appendChild(nameCell);
-      treeitem.appendChild(treerow);
-      treechildren.appendChild(treeitem);
+      this._tree.invalidate();
     }.bind(this);
 
     // Add loading row while we're loading a group list
     var loadingLabel = Zotero.getString("zotero.preferences.sync.librariesToSync.loadingLibraries");
     addRow(loadingLabel, "loading", false, false);
 
-    var apiKey = yield Zotero.Sync.Data.Local.getAPIKey();
+    var apiKey = await Zotero.Sync.Data.Local.getAPIKey();
     var client = Zotero.Sync.Runner.getAPIClient({ apiKey });
     var groups = [];
     try {
       // Load up remote groups
-      var keyInfo = yield Zotero.Sync.Runner.checkAccess(client, { timeout: 5000 });
-      groups = yield client.getGroups(keyInfo.userID);
+      var keyInfo = await Zotero.Sync.Runner.checkAccess(client, { timeout: 5000 });
+      groups = await client.getGroups(keyInfo.userID);
     }
     catch (e) {
       // Connection problems
@@ -342,7 +377,8 @@ Zotero_Preferences.Sync = {
     }
 
     // Remove the loading row
-    treechildren.removeChild(treechildren.firstChild);
+    this._rows = [];
+    this._tree.invalidate();
 
     var librariesToSkip = JSON.parse(Zotero.Prefs.get('sync.librariesToSkip') || '[]');
     // Add default rows
@@ -356,8 +392,7 @@ Zotero_Preferences.Sync = {
     for (let group of groups) {
       addRow(group.data.name, "G" + group.id, librariesToSkip.indexOf("G" + group.id) == -1);
     }
-  }),
-
+  },
 
   updateStorageSettingsUI: Zotero.Promise.coroutine(function* () {
     this.unverifyStorageServer();
@@ -524,7 +559,7 @@ Zotero_Preferences.Sync = {
     try {
       yield controller.checkServer({
         // Get the XMLHttpRequest for possible cancelling
-        onRequest: r => request = r });
+        onRequest: (r) => request = r });
 
 
       success = true;
@@ -595,17 +630,17 @@ Zotero_Preferences.Sync = {
     //
     var libraryMenu = document.getElementById('sync-reset-library-menu');
     // Some options need to be disabled when certain libraries are selected
-    libraryMenu.onchange = event => {
+    libraryMenu.onchange = (event) => {
       this.onResetLibraryChange(parseInt(event.target.value));
     };
     this.onResetLibraryChange(Zotero.Libraries.userLibraryID);
     var libraries = Zotero.Libraries.getAll().
-    filter(x => x.libraryType == 'user' || x.libraryType == 'group');
+    filter((x) => x.libraryType == 'user' || x.libraryType == 'group');
     Zotero.Utilities.Internal.buildLibraryMenuHTML(libraryMenu, libraries);
     // Disable read-only libraries, at least until there are options that make sense for those
     Array.from(libraryMenu.querySelectorAll('option')).
-    filter(x => x.getAttribute('data-editable') == 'false').
-    forEach(x => x.disabled = true);
+    filter((x) => x.getAttribute('data-editable') == 'false').
+    forEach((x) => x.disabled = true);
 
     var list = document.getElementById('sync-reset-list');
     for (let li of document.querySelectorAll('#sync-reset-list li')) {
@@ -657,111 +692,111 @@ Zotero_Preferences.Sync = {
 
     var libraryID = parseInt(
     Array.from(document.querySelectorAll('#sync-reset-library-menu option')).
-    filter(x => x.selected)[0].
+    filter((x) => x.selected)[0].
     value);
 
     var library = Zotero.Libraries.get(libraryID);
     var action = Array.from(document.querySelectorAll('#sync-reset-list input[name=sync-reset-radiogroup]')).
-    filter(x => x.checked)[0].
+    filter((x) => x.checked)[0].
     getAttribute('value');
 
     switch (action) {
       /*case 'full-sync':
-                     	var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING)
-                     		+ (ps.BUTTON_POS_1) * (ps.BUTTON_TITLE_CANCEL)
-                     		+ ps.BUTTON_POS_1_DEFAULT;
-                     	var index = ps.confirmEx(
-                     		null,
-                     		Zotero.getString('general.warning'),
-                     		// TODO: localize
-                     		"On the next sync, Zotero will compare all local and remote data and merge any "
-                     			+ "data that does not exist in both locations.\n\n"
-                     			+ "This option is not necessary during normal usage and should "
-                     			+ "generally be used only to troubleshoot specific issues as recommended "
-                     			+ "by Zotero support staff.",
-                     		buttonFlags,
-                     		Zotero.getString('general.reset'),
-                     		null, null, null, {}
-                     	);
-                     	
-                     	switch (index) {
-                     	case 0:
-                     		let libraries = Zotero.Libraries.getAll().filter(library => library.syncable);
-                     		await Zotero.DB.executeTransaction(function* () {
-                     			for (let library of libraries) {
-                     				library.libraryVersion = -1;
-                     				yield library.save();
-                     			}
-                     		});
-                     		break;
-                     		
-                     		// Cancel
-                     	case 1:
-                     		return;
-                     	}
-                     	
-                     	break;
-                     
-                     case 'restore-from-server':
-                     	var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING)
-                     						+ (ps.BUTTON_POS_1) * (ps.BUTTON_TITLE_CANCEL)
-                     						+ ps.BUTTON_POS_1_DEFAULT;
-                     	var index = ps.confirmEx(
-                     		null,
-                     		Zotero.getString('general.warning'),
-                     		Zotero.getString('zotero.preferences.sync.reset.restoreFromServer', account),
-                     		buttonFlags,
-                     		Zotero.getString('zotero.preferences.sync.reset.replaceLocalData'),
-                     		null, null, null, {}
-                     	);
-                     	
-                     	switch (index) {
-                     		case 0:
-                     			// TODO: better error handling
-                     			
-                     			// Verify username and password
-                     			var callback = async function () {
-                     				Zotero.Schema.stopRepositoryTimer();
-                     				Zotero.Sync.Runner.clearSyncTimeout();
-                     				
-                     				Zotero.DB.skipBackup = true;
-                     				
-                     				await Zotero.File.putContentsAsync(
-                     					OS.Path.join(Zotero.DataDirectory.dir, 'restore-from-server'),
-                     					''
-                     				);
-                     				
-                     				var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING);
-                     				var index = ps.confirmEx(
-                     					null,
-                     					Zotero.getString('general.restartRequired'),
-                     					Zotero.getString('zotero.preferences.sync.reset.restartToComplete'),
-                     					buttonFlags,
-                     					Zotero.getString('general.restartNow'),
-                     					null, null, null, {}
-                     				);
-                     				
-                     				var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
-                     						.getService(Components.interfaces.nsIAppStartup);
-                     				appStartup.quit(Components.interfaces.nsIAppStartup.eRestart | Components.interfaces.nsIAppStartup.eAttemptQuit);
-                     			};
-                     			
-                     			// TODO: better way of checking for an active session?
-                     			if (Zotero.Sync.Server.sessionIDComponent == 'sessionid=') {
-                     				Zotero.Sync.Server.login()
-                     				.then(callback)
-                     				.done();
-                     			}
-                     			else {
-                     				callback();
-                     			}
-                     			break;
-                     		
-                     		// Cancel
-                     		case 1:
-                     			return;
-                     	}
-                     	break;*/
+      	var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING)
+      		+ (ps.BUTTON_POS_1) * (ps.BUTTON_TITLE_CANCEL)
+      		+ ps.BUTTON_POS_1_DEFAULT;
+      	var index = ps.confirmEx(
+      		null,
+      		Zotero.getString('general.warning'),
+      		// TODO: localize
+      		"On the next sync, Zotero will compare all local and remote data and merge any "
+      			+ "data that does not exist in both locations.\n\n"
+      			+ "This option is not necessary during normal usage and should "
+      			+ "generally be used only to troubleshoot specific issues as recommended "
+      			+ "by Zotero support staff.",
+      		buttonFlags,
+      		Zotero.getString('general.reset'),
+      		null, null, null, {}
+      	);
+      	
+      	switch (index) {
+      	case 0:
+      		let libraries = Zotero.Libraries.getAll().filter(library => library.syncable);
+      		await Zotero.DB.executeTransaction(function* () {
+      			for (let library of libraries) {
+      				library.libraryVersion = -1;
+      				yield library.save();
+      			}
+      		});
+      		break;
+      		
+      		// Cancel
+      	case 1:
+      		return;
+      	}
+      	
+      	break;
+      
+      case 'restore-from-server':
+      	var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING)
+      						+ (ps.BUTTON_POS_1) * (ps.BUTTON_TITLE_CANCEL)
+      						+ ps.BUTTON_POS_1_DEFAULT;
+      	var index = ps.confirmEx(
+      		null,
+      		Zotero.getString('general.warning'),
+      		Zotero.getString('zotero.preferences.sync.reset.restoreFromServer', account),
+      		buttonFlags,
+      		Zotero.getString('zotero.preferences.sync.reset.replaceLocalData'),
+      		null, null, null, {}
+      	);
+      	
+      	switch (index) {
+      		case 0:
+      			// TODO: better error handling
+      			
+      			// Verify username and password
+      			var callback = async function () {
+      				Zotero.Schema.stopRepositoryTimer();
+      				Zotero.Sync.Runner.clearSyncTimeout();
+      				
+      				Zotero.DB.skipBackup = true;
+      				
+      				await Zotero.File.putContentsAsync(
+      					OS.Path.join(Zotero.DataDirectory.dir, 'restore-from-server'),
+      					''
+      				);
+      				
+      				var buttonFlags = (ps.BUTTON_POS_0) * (ps.BUTTON_TITLE_IS_STRING);
+      				var index = ps.confirmEx(
+      					null,
+      					Zotero.getString('general.restartRequired'),
+      					Zotero.getString('zotero.preferences.sync.reset.restartToComplete'),
+      					buttonFlags,
+      					Zotero.getString('general.restartNow'),
+      					null, null, null, {}
+      				);
+      				
+      				var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
+      						.getService(Components.interfaces.nsIAppStartup);
+      				appStartup.quit(Components.interfaces.nsIAppStartup.eRestart | Components.interfaces.nsIAppStartup.eAttemptQuit);
+      			};
+      			
+      			// TODO: better way of checking for an active session?
+      			if (Zotero.Sync.Server.sessionIDComponent == 'sessionid=') {
+      				Zotero.Sync.Server.login()
+      				.then(callback)
+      				.done();
+      			}
+      			else {
+      				callback();
+      			}
+      			break;
+      		
+      		// Cancel
+      		case 1:
+      			return;
+      	}
+      	break;*/
 
       case 'restore-to-server':
         var buttonFlags = ps.BUTTON_POS_0 * ps.BUTTON_TITLE_IS_STRING +
