@@ -26,22 +26,50 @@
 Components.utils.import("resource://gre/modules/Services.jsm");
 
 /**
-                                                                 * This object contains the various functions for the interface
-                                                                 */
+ * This object contains the various functions for the interface
+ */
 const ZoteroStandalone = new function () {
   const FONT_SIZES = ["1.0", "1.15", "1.3", "1.5", "1.7", "1.9", "2.1"];
   //const NOTE_FONT_SIZES = ["11", "12", "13", "14", "18", "24", "36", "48", "64", "72", "96"];
   const NOTE_FONT_SIZE_DEFAULT = "12";
 
   /**
-                                        * Run when standalone window first opens
-                                        */
+   * Run when standalone window first opens
+   */
   this.onLoad = function () {
     // Fix window without menubar/titlebar when Zotero is closed in full-screen mode in OS X 10.11+
     if (Zotero.isMac && window.document.documentElement.getAttribute('sizemode') == 'fullscreen') {
       window.document.documentElement.setAttribute('sizemode', 'normal');
     }
 
+    // Create tab bar by default
+    if (Zotero.isMac) {
+      document.documentElement.setAttribute('drawintitlebar', true);
+      document.documentElement.setAttribute('tabsintitlebar', true);
+      document.documentElement.setAttribute('chromemargin', '0,-1,-1,-1');
+    }
+
+    this.switchMenuType('library');
+    this._notifierID = Zotero.Notifier.registerObserver(
+    {
+      notify: async (action, type, ids, extraData) => {
+        if (action == 'select') {
+          // Reader doesn't have tabID yet
+          setTimeout(async () => {
+            // Item and other things might not be loaded yet when reopening tabs
+            await Zotero.Schema.schemaUpdatePromise;
+            this.updateQuickCopyOptions();
+          }, 0);
+          // "library" or "reader"
+          this.switchMenuType(extraData[ids[0]].type);
+          setTimeout(() => ZoteroPane.updateToolbarPosition(), 0);
+        }
+      } },
+
+    ['tab'],
+    'tab');
+
+
     Zotero.Promise.try(function () {
       if (!Zotero) {
         throw true;
@@ -89,15 +117,35 @@ const ZoteroStandalone = new function () {
       window.close();
       return;
     });
+
+    // Switch to library tab if dragging over one or more PDF files
+    window.addEventListener('dragover', function (event) {
+      // TODO: Consider allowing more (or all) file types, although shouldn't interfere with image dragging to note editor
+      if (Zotero_Tabs.selectedID != 'zotero-pane' &&
+      event.dataTransfer.items &&
+      event.dataTransfer.items.length &&
+      !Array.from(event.dataTransfer.items).find((x) => x.type != 'application/pdf')) {
+        Zotero_Tabs.select('zotero-pane');
+      }
+    }, true);
   };
 
+  this.switchMenuType = function (type) {
+    document.querySelectorAll('.menu-type-library, .menu-type-reader').forEach((el) => el.hidden = true);
+    document.querySelectorAll('.menu-type-' + type).forEach((el) => el.hidden = false);
+  };
+
+  this.onReaderCmd = function (cmd) {
+    let reader = Zotero.Reader.getByTabID(Zotero_Tabs.selectedID);
+    reader.menuCmd(cmd);
+  };
 
   this.onFileMenuOpen = function () {
     var active = false;
     try {
       let zp = Zotero.getActiveZoteroPane();
       if (zp) {
-        active = !!zp.getSelectedItems().filter(item => {
+        active = !!zp.getSelectedItems().filter((item) => {
           return item.isAttachment() ||
           item.isRegularItem() && item.getAttachments().length;
         }).length;
@@ -105,12 +153,68 @@ const ZoteroStandalone = new function () {
     }
     catch (e) {}
     this.updateMenuItemEnabled('manage-attachments-menu', active);
+
+    // PDF annotation transfer ("Import Annotation"/"Store Annotations in File")
+    let reader = Zotero.Reader.getByTabID(Zotero_Tabs.selectedID);
+    if (reader) {
+      let item = Zotero.Items.get(reader.itemID);
+      let library = Zotero.Libraries.get(item.libraryID);
+      if (item
+      // Don't allow annotation transfer in group libraries
+      && library.libraryType == 'user' &&
+      library.editable &&
+      !(item.deleted || item.parentItem && item.parentItem.deleted)) {
+        let annotations = item.getAnnotations();
+        let canTransferFromPDF = annotations.find((x) => x.annotationIsExternal);
+        this.updateMenuItemEnabled('menu_transferFromPDF', canTransferFromPDF);
+      } else
+      {
+        this.updateMenuItemEnabled('menu_transferFromPDF', false);
+      }
+    }
+
+    // TEMP: Quick implementation
+    try {
+      let menuitem = document.getElementById('menu_export_files');
+      let sep = menuitem.nextSibling;
+
+      let zp = Zotero.getActiveZoteroPane();
+      if (zp && !reader) {
+        let numFiles = zp.getSelectedItems().reduce((num, item) => {
+          if (item.isPDFAttachment()) {
+            return num + 1;
+          }
+          if (item.isRegularItem()) {
+            return num + item.numPDFAttachments();
+          }
+          return num;
+        }, 0);
+        if (numFiles) {
+          menuitem.hidden = false;
+          sep.hidden = false;
+          menuitem.label = Zotero.getString(
+          'pane.items.menu.exportPDF' + (numFiles == 1 ? '' : '.multiple'));
+
+        } else
+        {
+          menuitem.hidden = true;
+          sep.hidden = true;
+        }
+      } else
+      {
+        menuitem.hidden = true;
+        sep.hidden = true;
+      }
+    }
+    catch (e) {
+      Zotero.logError(e);
+    }
   };
 
 
   /**
-      * Builds new item menu
-      */
+   * Builds new item menu
+   */
   this.buildNewItemMenu = function () {
     var addMenu = document.getElementById('menu_NewItemPopup');
 
@@ -161,12 +265,12 @@ const ZoteroStandalone = new function () {
     try {
       let zp = Zotero.getActiveZoteroPane();
       if (zp) {
-        active = !!zp.getSelectedItems().filter(item => {
+        active = !!zp.getSelectedItems().filter((item) => {
           return item.isLinkedFileAttachment() ||
           item.isRegularItem() &&
           item.getAttachments().
-          map(id => Zotero.Items.get(id)).
-          some(att => att.isLinkedFileAttachment());
+          map((id) => Zotero.Items.get(id)).
+          some((att) => att.isLinkedFileAttachment());
         }).length;
       }
     }
@@ -193,25 +297,45 @@ const ZoteroStandalone = new function () {
 
 
   this.updateQuickCopyOptions = function () {
-    var selected = false;
-    try {
-      selected = Zotero.getActiveZoteroPane().
-      getSelectedItems().
-      filter(item => item.isRegularItem()).
-      length;
+    var selected = [];
+
+    let win = Zotero.getMainWindow();
+    if (win) {
+      if (win.Zotero_Tabs.selectedID == 'zotero-pane') {
+        try {
+          selected = win.ZoteroPane.getSelectedItems();
+        }
+        catch (e) {
+        }
+        win.ZoteroPane.updateQuickCopyCommands(selected);
+      } else
+      {
+        let reader = Zotero.Reader.getByTabID(win.Zotero_Tabs.selectedID);
+        if (reader) {
+          let item = Zotero.Items.get(reader.itemID);
+          selected = item.parentItem && [item.parentItem] || [];
+          item = item.parentItem || item;
+          win.ZoteroPane.updateQuickCopyCommands([item]);
+        }
+      }
     }
-    catch (e) {}
 
     var format = Zotero.QuickCopy.getFormatFromURL(Zotero.QuickCopy.lastActiveURL);
+    var exportingNotes = selected.every((item) => item.isNote() || item.isAttachment());
+    if (exportingNotes) {
+      format = Zotero.QuickCopy.getNoteFormat();
+    }
     format = Zotero.QuickCopy.unserializeSetting(format);
 
     var copyCitation = document.getElementById('menu_copyCitation');
     var copyBibliography = document.getElementById('menu_copyBibliography');
     var copyExport = document.getElementById('menu_copyExport');
+    var copyNote = document.getElementById('menu_copyNote');
 
-    copyCitation.hidden = !selected || format.mode != 'bibliography';
-    copyBibliography.hidden = !selected || format.mode != 'bibliography';
-    copyExport.hidden = !selected || format.mode != 'export';
+    copyCitation.hidden = !selected.length || format.mode != 'bibliography';
+    copyBibliography.hidden = !selected.length || format.mode != 'bibliography';
+    copyExport.hidden = !selected.length || format.mode != 'export' || exportingNotes;
+    copyNote.hidden = !selected.length || format.mode != 'export' || !exportingNotes;
     if (format.mode == 'export') {
       try {
         let obj = Zotero.Translators.get(format.id);
@@ -232,7 +356,57 @@ const ZoteroStandalone = new function () {
   };
 
 
+  this.onGoMenuOpen = function () {
+    var keyBack = document.getElementById('key_back');
+    var keyForward = document.getElementById('key_forward');
+
+    if (Zotero.isMac) {
+      keyBack.setAttribute('key', '[');
+      keyBack.setAttribute('modifiers', 'meta');
+      keyForward.setAttribute('key', ']');
+      keyForward.setAttribute('modifiers', 'meta');
+    } else
+    {
+      keyBack.setAttribute('keycode', 'VK_LEFT');
+      keyBack.setAttribute('modifiers', 'alt');
+      keyForward.setAttribute('keycode', 'VK_RIGHT');
+      keyForward.setAttribute('modifiers', 'alt');
+    }
+
+    // `key` attribute needs to be dynamically set for `menuitem` when
+    // the key changes after DOM initialization
+    var menuItemBack = document.getElementById('go-menuitem-back');
+    var menuItemForward = document.getElementById('go-menuitem-forward');
+    menuItemBack.setAttribute('key', 'key_back');
+    menuItemForward.setAttribute('key', 'key_forward');
+
+    var reader = Zotero.Reader.getByTabID(Zotero_Tabs.selectedID);
+    if (reader) {
+      this.updateMenuItemEnabled('go-menuitem-first-page', reader.allowNavigateFirstPage());
+      this.updateMenuItemEnabled('go-menuitem-last-page', reader.allowNavigateLastPage());
+      this.updateMenuItemEnabled('go-menuitem-back', reader.allowNavigateBack());
+      this.updateMenuItemEnabled('go-menuitem-forward', reader.allowNavigateForward());
+    }
+  };
+
+
   this.onViewMenuOpen = function () {
+    // PDF Reader
+    var reader = Zotero.Reader.getByTabID(Zotero_Tabs.selectedID);
+    if (reader) {
+      var { state } = reader;
+      this.updateMenuItemCheckmark('view-menuitem-vertical-scrolling', state.scrollMode == 0);
+      this.updateMenuItemCheckmark('view-menuitem-horizontal-scrolling', state.scrollMode == 1);
+      this.updateMenuItemCheckmark('view-menuitem-wrapped-scrolling', state.scrollMode == 2);
+      this.updateMenuItemCheckmark('view-menuitem-no-spreads', state.spreadMode == 0);
+      this.updateMenuItemCheckmark('view-menuitem-odd-spreads', state.spreadMode == 1);
+      this.updateMenuItemCheckmark('view-menuitem-even-spreads', state.spreadMode == 2);
+      this.updateMenuItemCheckmark('view-menuitem-hand-tool', reader.isHandToolActive());
+      this.updateMenuItemCheckmark('view-menuitem-zoom-auto', reader.isZoomAutoActive());
+      this.updateMenuItemCheckmark('view-menuitem-zoom-page-width', reader.isZoomPageWidthActive());
+      this.updateMenuItemCheckmark('view-menuitem-zoom-page-height', reader.isZoomPageHeightActive());
+    }
+
     // Layout mode
     var mode = Zotero.Prefs.get('layout');
     this.updateMenuItemCheckmark('view-menuitem-standard', mode != 'stacked');
@@ -307,9 +481,9 @@ const ZoteroStandalone = new function () {
         }
         // Hide
         else {
-            document.getElementById('zotero-collections-splitter').setAttribute('state', 'collapsed');
-            collectionsPane.setAttribute('collapsed', true);
-          }
+          document.getElementById('zotero-collections-splitter').setAttribute('state', 'collapsed');
+          collectionsPane.setAttribute('collapsed', true);
+        }
         ZoteroPane.updateToolbarPosition();
         break;
 
@@ -322,9 +496,9 @@ const ZoteroStandalone = new function () {
         }
         // Hide
         else {
-            document.getElementById('zotero-items-splitter').setAttribute('state', 'collapsed');
-            itemPane.setAttribute('collapsed', true);
-          }
+          document.getElementById('zotero-items-splitter').setAttribute('state', 'collapsed');
+          itemPane.setAttribute('collapsed', true);
+        }
         ZoteroPane.updateToolbarPosition();
         break;
 
@@ -345,17 +519,16 @@ const ZoteroStandalone = new function () {
         break;
 
       /*case 'note-font-size-bigger':
-               	increaseFontSize('note.fontSize', NOTE_FONT_SIZES);
-               	break;
-               
-               case 'note-font-size-smaller':
-               	decreaseFontSize('note.fontSize', NOTE_FONT_SIZES);
-               	break;
-               */
+      	increaseFontSize('note.fontSize', NOTE_FONT_SIZES);
+      	break;
+      
+      case 'note-font-size-smaller':
+      	decreaseFontSize('note.fontSize', NOTE_FONT_SIZES);
+      	break;
+      */
 
       case 'note-font-size-reset':
         Zotero.Prefs.clear('note.fontSize');
-        this.promptForRestart();
         break;
 
       case 'recursive-collections':
@@ -422,7 +595,6 @@ const ZoteroStandalone = new function () {
   this.updateNoteFontSize = function (event) {
     var size = event.originalTarget.getAttribute('label');
     Zotero.Prefs.set('note.fontSize', size);
-    this.promptForRestart();
   };
 
 
@@ -475,8 +647,8 @@ const ZoteroStandalone = new function () {
   };
 
   /**
-      * Handles help menu requests
-      */
+   * Handles help menu requests
+   */
   this.openHelp = function (type) {
     Components.utils.import("resource://zotero/config.js");
 
@@ -499,16 +671,17 @@ const ZoteroStandalone = new function () {
   };
 
   /**
-      * Checks for updates
-      */
+   * Checks for updates
+   */
   this.checkForUpdates = function () {
     window.open('chrome://mozapps/content/update/updates.xul', 'updateChecker', 'chrome,centerscreen');
   };
 
   /**
-      * Called before standalone window is closed
-      */
+   * Called before standalone window is closed
+   */
   this.onUnload = function () {
+    Zotero.Notifier.unregisterObserver(this._notifierID);
     ZoteroPane.destroy();
   };
 }();
@@ -769,7 +942,7 @@ const gXPInstallObserver = {
         Zotero.getString("standalone.addonInstallationFailed.body", installs[0].name));
         break;
       /*case "addon-install-started":
-               case "addon-install-complete":*/}
+      case "addon-install-complete":*/}
 
   } };
 
@@ -782,7 +955,7 @@ function openUILinkIn(url) {
 // Support context menus on HTML text boxes
 //
 // Adapted from editMenuOverlay.js in Fx68
-window.addEventListener("contextmenu", e => {
+window.addEventListener("contextmenu", (e) => {
   const HTML_NS = "http://www.w3.org/1999/xhtml";
   let needsContextMenu =
   e.target.ownerDocument == document &&
