@@ -1,23 +1,124 @@
 "use strict";
 
 /**
-               * Utility functions for dealing with citations
-               * @namespace
-               */
+ * Utility functions for dealing with citations
+ * @namespace
+ */
 Zotero.Cite = {
   /**
-                 * Locator labels
-                 */
-  "labels": ["page", "book", "chapter", "column", "figure", "folio",
-  "issue", "line", "note", "opus", "paragraph", "part", "section", "sub verbo",
-  "volume", "verse"],
+   * Locator labels
+   */
+  "labels": [
+  "act",
+  "appendix",
+  "article-locator",
+  "book",
+  "canon",
+  "chapter",
+  "column",
+  "elocation",
+  "equation",
+  "figure",
+  "folio",
+  "issue",
+  "line",
+  "note",
+  "opus",
+  "page",
+  "paragraph",
+  "part",
+  "rule",
+  "scene",
+  "section",
+  "sub-verbo",
+  "table",
+  //"timestamp",
+  "title-locator",
+  "verse",
+  "volume"],
+
+
+  _locatorStrings: new Map(),
 
   /**
-                       * Remove specified item IDs in-place from a citeproc-js bibliography object returned
-                       * by makeBibliography()
-                       * @param {bib} citeproc-js bibliography object
-                       * @param {Set} itemsToRemove Set of items to remove
-                       */
+   * Get localized string for locator
+   *
+   * @param {String} locator - Locator name (e.g., 'book')
+   * @return {String} - Localized string (e.g., 'Livre')
+   */
+  getLocatorString: function (locator) {
+    // 'timestamp' not included in CSL locales
+    if (locator == 'timestamp') {
+      return Zotero.getString('citation.locator.timestamp');
+    }
+
+    // Get the best CSL locale for the current Zotero locale
+    var cslLocale = Zotero.Utilities.Internal.resolveLocale(
+    Zotero.locale,
+    Object.keys(Zotero.Styles.locales));
+
+
+    // If locator strings are already cached for the current locale, use that
+    if (this._locatorStrings.has(cslLocale)) {
+      return this._locatorStrings.get(cslLocale).get(locator);
+    }
+    var map = new Map();
+    this._locatorStrings.set(cslLocale, map);
+
+    var localeXML = Zotero.Cite.Locale.get(cslLocale);
+    var parser;
+    if (Zotero.platformMajorVersion > 60) {
+      parser = new DOMParser();
+    } else
+    {
+      parser = Components.classes["@mozilla.org/xmlextras/domparser;1"].
+      createInstance(Components.interfaces.nsIDOMParser);
+    }
+    var doc = parser.parseFromString(localeXML, 'text/xml');
+    var englishDoc;
+    // Cache all locators for the current locale
+    for (let locator of this.labels) {
+      let elem = doc.querySelector(`term[name="${locator}"]:not([form="short"]) > single`);
+      if (!elem) {
+        // If locator not found, get from the U.S. English locale
+        if (cslLocale != 'en-US') {
+          Zotero.logError(`Locator '${locator}' not found in ${cslLocale} locale -- trying en-US`);
+          if (!englishDoc) {
+            englishDoc = parser.parseFromString(Zotero.Cite.Locale.get('en-US'), 'text/xml');
+          }
+          elem = englishDoc.querySelector(`term[name="${locator}"]:not([form="short"]) > single`);
+          if (!elem) {
+            Zotero.logError(`Locator '${locator}' not found in en-US locale -- using name`);
+          }
+        } else
+        {
+          Zotero.logError(`Locator '${locator}' not found in en-US locale -- using name`);
+        }
+        // If still not found, use the locator name directly
+        if (!elem) {
+          map.set(locator, Zotero.Utilities.capitalize(locator));
+          continue;
+        }
+      }
+      // If <single> is empty, use the locator name directly
+      let str = elem.textContent;
+      if (!str) {
+        Zotero.logError(`Locator '${locator}' is empty in ${cslLocale} locale -- using name`);
+        map.set(locator, Zotero.Utilities.capitalize(locator));
+        continue;
+      }
+      map.set(locator, Zotero.Utilities.capitalize(str));
+    }
+
+    return map.get(locator);
+  },
+
+  /**
+   * Remove specified item IDs in-place from a citeproc-js bibliography object returned
+   * by makeBibliography()
+   * @param {bib} citeproc-js bibliography object
+   * @param {Set} itemsToRemove Set of items to remove
+   */
   "removeFromBibliography": function (bib, itemsToRemove) {
     var removeItems = [];
     for (let i in bib[0].entry_ids) {
@@ -35,11 +136,11 @@ Zotero.Cite = {
   },
 
   /**
-      * Convert formatting data from citeproc-js bibliography object into explicit format
-      * parameters for RTF or word processors
-      * @param {bib} citeproc-js bibliography object
-      * @return {Object} Bibliography style parameters.
-      */
+   * Convert formatting data from citeproc-js bibliography object into explicit format
+   * parameters for RTF or word processors
+   * @param {bib} citeproc-js bibliography object
+   * @return {Object} Bibliography style parameters.
+   */
   "getBibliographyFormatParameters": function getBibliographyFormatParameters(bib) {
     var bibStyle = { "tabStops": [], "indent": 0, "firstLineIndent": 0,
       "lineSpacing": 240 * bib[0].linespacing,
@@ -64,16 +165,16 @@ Zotero.Cite = {
   },
 
   /**
-      * Makes a formatted bibliography, if the style defines one; otherwise makes a 
-      * formatted list of items
-      * @param {Zotero.Style} style The style to use
-      * @param {Zotero.Item[]} items An array of items
-      * @param {String} format The format of the output (html, text, or rtf)
-      * @return {String} Bibliography or item list in specified format
-      */
+   * Makes a formatted bibliography, if the style defines one; otherwise makes a 
+   * formatted list of items
+   * @param {Zotero.Style} style The style to use
+   * @param {Zotero.Item[]} items An array of items
+   * @param {String} format The format of the output (html, text, or rtf)
+   * @return {String} Bibliography or item list in specified format
+   */
   "makeFormattedBibliographyOrCitationList": function (cslEngine, items, format, asCitationList) {
     cslEngine.setOutputFormat(format);
-    cslEngine.updateItems(items.map(item => item.id));
+    cslEngine.updateItems(items.map((item) => item.id));
 
     if (!asCitationList) {
       var bibliography = Zotero.Cite.makeFormattedBibliography(cslEngine, format);
@@ -127,13 +228,12 @@ Zotero.Cite = {
   },
 
   /**
-      * Makes a formatted bibliography
-      * @param {Zotero.Style} style The style
-      * @param {String} format The format of the output (html, text, or rtf)
-      * @return {String} Bibliography in specified format
-      */
+   * Makes a formatted bibliography
+   * @param {Zotero.Style} style The style
+   * @param {String} format The format of the output (html, text, or rtf)
+   * @return {String} Bibliography in specified format
+   */
   "makeFormattedBibliography": function makeFormattedBibliography(cslEngine, format) {
-    cslEngine.setOutputFormat(format);
     var bib = cslEngine.makeBibliography();
     if (!bib) return false;
 
@@ -206,8 +306,8 @@ Zotero.Cite = {
         }
         // If only one field, apply hanging indent on root
         else if (!multiField) {
-            style += "margin-left: 2em; text-indent:-2em;";
-          }
+          style += "margin-left: 2em; text-indent:-2em;";
+        }
       }
 
       if (style) div.setAttribute("style", style);
@@ -288,16 +388,16 @@ Zotero.Cite = {
   },
 
   /**
-      * Get an item by ID, either by retrieving it from the library or looking for the document it
-      * belongs to.
-      * @param {String|Number|Array} id
-      * @return {Zotero.Item} item
-      */
+   * Get an item by ID, either by retrieving it from the library or looking for the document it
+   * belongs to.
+   * @param {String|Number|Array} id
+   * @return {Zotero.Item} item
+   */
   "getItem": function getItem(id) {
     var slashIndex;
 
     if (id instanceof Array) {
-      return id.map(anId => Zotero.Cite.getItem(anId));
+      return id.map((anId) => Zotero.Cite.getItem(anId));
     } else if (typeof id === "string" && (slashIndex = id.indexOf("/")) !== -1) {
       var sessionID = id.substr(0, slashIndex),
       session = Zotero.Integration.sessions[sessionID],
@@ -318,127 +418,21 @@ Zotero.Cite = {
   },
 
   extraToCSL: function (extra) {
-    return extra.replace(/^([A-Za-z \-]+)(:\s*.+)/gm, function (_, field, value) {
-      var originalField = field;
-      var field = field.toLowerCase().replace(/ /g, '-');
-      // Fields from https://aurimasv.github.io/z2csl/typeMap.xml
-      switch (field) {
-        // Standard fields
-        case 'abstract':
-        case 'accessed':
-        case 'annote':
-        case 'archive':
-        case 'archive-place':
-        case 'author':
-        case 'authority':
-        case 'call-number':
-        case 'chapter-number':
-        case 'citation-label':
-        case 'citation-number':
-        case 'collection-editor':
-        case 'collection-number':
-        case 'collection-title':
-        case 'composer':
-        case 'container':
-        case 'container-author':
-        case 'container-title':
-        case 'container-title-short':
-        case 'dimensions':
-        case 'director':
-        case 'edition':
-        case 'editor':
-        case 'editorial-director':
-        case 'event':
-        case 'event-date':
-        case 'event-place':
-        case 'first-reference-note-number':
-        case 'genre':
-        case 'illustrator':
-        case 'interviewer':
-        case 'issue':
-        case 'issued':
-        case 'jurisdiction':
-        case 'keyword':
-        case 'language':
-        case 'locator':
-        case 'medium':
-        case 'note':
-        case 'number':
-        case 'number-of-pages':
-        case 'number-of-volumes':
-        case 'original-author':
-        case 'original-date':
-        case 'original-publisher':
-        case 'original-publisher-place':
-        case 'original-title':
-        case 'page':
-        case 'page-first':
-        case 'publisher':
-        case 'publisher-place':
-        case 'recipient':
-        case 'references':
-        case 'reviewed-author':
-        case 'reviewed-title':
-        case 'scale':
-        case 'section':
-        case 'source':
-        case 'status':
-        case 'submitted':
-        case 'title':
-        case 'title-short':
-        case 'translator':
-        case 'type':
-        case 'version':
-        case 'volume':
-        case 'year-suffix':
-          break;
-
-        // Uppercase fields
-        case 'doi':
-        case 'isbn':
-        case 'issn':
-        case 'pmcid':
-        case 'pmid':
-        case 'url':
-          field = field.toUpperCase();
-          break;
-
-        // Weirdo
-        case 'archive-location':
-          field = 'archive_location';
-          break;
-
-        default:
-          // See if this is a Zotero field written out (e.g., "Publication Title"), and if so
-          // convert to its associated CSL field
-          var zoteroField = originalField.replace(/ ([A-Z])/, '$1');
-          // If second character is lowercase (so not an acronym), lowercase first letter too
-          if (zoteroField[1] == zoteroField[1].toLowerCase()) {
-            zoteroField = zoteroField[0].toLowerCase() + zoteroField.substr(1);
-          }
-          if (Zotero.Schema.CSL_FIELD_MAPPINGS_REVERSE[zoteroField]) {
-            field = Zotero.Schema.CSL_FIELD_MAPPINGS_REVERSE[zoteroField];
-          }
-          // Don't change other lines
-          else {
-              field = originalField;
-            }}
-
-      return field + value;
-    });
+    Zotero.debug(`Zotero.Cite.extraToCSL() is deprecated -- use Zotero.Utilities.Item.extraToCSL() instead`);
+    return Zotero.Utilities.Item.extraToCSL(extra);
   } };
 
 
 /**
-        * Get a CSL abbreviation in the format expected by citeproc-js
-        */
+ * Get a CSL abbreviation in the format expected by citeproc-js
+ */
 Zotero.Cite.getAbbreviation = new function () {
   var abbreviations,
   abbreviationCategories;
 
   /**
-                           * Initialize abbreviations database.
-                           */
+   * Initialize abbreviations database.
+   */
   function init() {
     if (!abbreviations) loadAbbreviations();
   }
@@ -477,8 +471,8 @@ Zotero.Cite.getAbbreviation = new function () {
   }
 
   /**
-     * Normalizes a key
-     */
+   * Normalizes a key
+   */
   function normalizeKey(key) {
     // Strip periods, normalize spacing, and convert to lowercase
     return key.toString().
@@ -491,8 +485,8 @@ Zotero.Cite.getAbbreviation = new function () {
   }
 
   /**
-     * Replace getAbbreviation on citeproc-js with our own handler.
-     */
+   * Replace getAbbreviation on citeproc-js with our own handler.
+   */
   return function getAbbreviation(listname, obj, jurisdiction, category, key) {
     init();
 
@@ -596,13 +590,13 @@ Zotero.Cite.getAbbreviation = new function () {
 }();
 
 /**
-      * citeproc-js system object
-      *
-      * @class
-      * @param {Object} options
-      * @param {Boolean} [options.automaticJournalAbbreviations]
-      * @param {Boolean} [options.uppercaseSubtitles]
-      */
+ * citeproc-js system object
+ *
+ * @class
+ * @param {Object} options
+ * @param {Boolean} [options.automaticJournalAbbreviations]
+ * @param {Boolean} [options.uppercaseSubtitles]
+ */
 Zotero.Cite.System = function ({ automaticJournalAbbreviations, uppercaseSubtitles }) {
   if (automaticJournalAbbreviations) {
     this.getAbbreviation = Zotero.Cite.getAbbreviation;
@@ -614,11 +608,11 @@ Zotero.Cite.System = function ({ automaticJournalAbbreviations, uppercaseSubtitl
 
 Zotero.Cite.System.prototype = {
   /**
-                                  * citeproc-js system function for getting items
-                                  * See http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#retrieveitem
-                                  * @param {String|Integer} item - Item ID, or string item for embedded citations
-                                  * @return {Object} citeproc-js item
-                                  */
+   * citeproc-js system function for getting items
+   * See http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#retrieveitem
+   * @param {String|Integer} item - Item ID, or string item for embedded citations
+   * @return {Object} citeproc-js item
+   */
   "retrieveItem": function retrieveItem(item) {
     var zoteroItem, slashIndex;
     if (typeof item === "object" && item !== null && item instanceof Zotero.Item) {
@@ -647,7 +641,7 @@ Zotero.Cite.System.prototype = {
       throw new Error("Zotero.Cite.System.retrieveItem called on non-item " + item);
     }
 
-    var cslItem = Zotero.Utilities.itemToCSLJSON(zoteroItem);
+    var cslItem = Zotero.Utilities.Item.itemToCSLJSON(zoteroItem);
 
     // TEMP: citeproc-js currently expects the id property to be the item DB id
     cslItem.id = zoteroItem.id;
@@ -668,11 +662,11 @@ Zotero.Cite.System.prototype = {
   },
 
   /**
-      * citeproc-js system function for getting locale
-      * See http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#retrieveLocale
-      * @param {String} lang Language to look for a locale for
-      * @return {String|Boolean} The locale as a string if it exists, or false if it doesn't
-      */
+   * citeproc-js system function for getting locale
+   * See http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#retrieveLocale
+   * @param {String} lang Language to look for a locale for
+   * @return {String|Boolean} The locale as a string if it exists, or false if it doesn't
+   */
   "retrieveLocale": function retrieveLocale(lang) {
     return Zotero.Cite.Locale.get(lang);
   } };
