@@ -37,7 +37,10 @@ Zotero.DBConnection = function (dbNameOrPath) {
   }
 
   this.MAX_BOUND_PARAMETERS = 999;
-  this.DB_CORRUPTION_STRING = "2152857611";
+  this.DB_CORRUPTION_STRINGS = [
+  "database disk image is malformed",
+  "2152857611"];
+
 
   Components.utils.import("resource://gre/modules/Sqlite.jsm", this);
 
@@ -78,10 +81,10 @@ Zotero.DBConnection = function (dbNameOrPath) {
   }
   // DB name in data directory
   else {
-      this._dbName = dbNameOrPath;
-      this._dbPath = Zotero.DataDirectory.getDatabase(dbNameOrPath);
-      this._externalDB = false;
-    }
+    this._dbName = dbNameOrPath;
+    this._dbPath = Zotero.DataDirectory.getDatabase(dbNameOrPath);
+    this._externalDB = false;
+  }
   this._shutdown = false;
   this._connection = null;
   this._transactionID = null;
@@ -116,6 +119,12 @@ Zotero.DBConnection = function (dbNameOrPath) {
 // Public methods
 //
 /////////////////////////////////////////////////////////////////
+Zotero.defineProperty(Zotero.DBConnection.prototype, 'path', {
+  get: function () {
+    return this._dbPath;
+  } });
+
+
 
 /**
  * Test a read-only connection to the database, throwing any errors that occur
@@ -140,8 +149,8 @@ Zotero.DBConnection.prototype.parseQueryAndParams = function (sql, params) {
   }
   // Otherwise, since we might make changes, only work on a copy of the array
   else {
-      params = params.concat();
-    }
+    params = params.concat();
+  }
 
   // Find placeholders
   if (params.length) {
@@ -202,8 +211,8 @@ Zotero.DBConnection.prototype.parseQueryAndParams = function (sql, params) {
         }
         // String
         else if (typeof params[i]['string'] != 'undefined') {
-            params[i] = params[i]['string'] + "";
-          }
+          params[i] = params[i]['string'] + "";
+        }
 
         continue;
       }
@@ -291,10 +300,10 @@ Zotero.DBConnection.prototype.removeCallback = function (type, id) {
 
 
 /*
-    * Used on shutdown to rollback all open transactions
-    *
-    * TODO: update or remove
-    */
+ * Used on shutdown to rollback all open transactions
+ *
+ * TODO: update or remove
+ */
 Zotero.DBConnection.prototype.rollbackAllTransactions = function () {
   if (this.transactionInProgress()) {
     var level = this._transactionNestingLevel;
@@ -312,7 +321,7 @@ Zotero.DBConnection.prototype.rollbackAllTransactions = function () {
 Zotero.DBConnection.prototype.getColumns = function (table) {
   return this.queryAsync("PRAGMA table_info(" + table + ")").
   then(function (rows) {
-    return rows.map(row => row.name);
+    return rows.map((row) => row.name);
   }).
   catch(function (e) {
     this._debug(e, 1);
@@ -322,15 +331,14 @@ Zotero.DBConnection.prototype.getColumns = function (table) {
 
 
 /**
-   * Find the next lowest numeric suffix for a value in table column
-   *
-   * For example, if "Untitled" and "Untitled 2" and "Untitled 4",
-   * returns "Untitled 3"
-   *
-   * If _name_ alone is available, returns that
-   **/
-Zotero.DBConnection.prototype.getNextName = Zotero.Promise.coroutine(function* (libraryID, table, field, name)
-{
+* Find the next lowest numeric suffix for a value in table column
+*
+* For example, if "Untitled" and "Untitled 2" and "Untitled 4",
+* returns "Untitled 3"
+*
+* If _name_ alone is available, returns that
+**/
+Zotero.DBConnection.prototype.getNextName = async function (libraryID, table, field, name) {
   Zotero.debug("WARNING: Zotero.DB.getNextName() is deprecated -- " +
   "use Zotero.Utilities.Internal.getNextName() instead", 2);
 
@@ -341,8 +349,8 @@ Zotero.DBConnection.prototype.getNextName = Zotero.Promise.coroutine(function* (
   var sql = "SELECT SUBSTR(" + field + ", " + (name.length + 1) + ") FROM " + table +
   " WHERE libraryID=? AND " + field + " LIKE ? ORDER BY " + field;
   var params = [libraryID, name + "%"];
-  var suffixes = yield this.columnQueryAsync(sql, params);
-  suffixes.filter(x => x.match(/^( [0-9]+)?$/));
+  var suffixes = await this.columnQueryAsync(sql, params);
+  suffixes.filter((x) => x.match(/^( [0-9]+)?$/));
 
   // If none found or first one has a suffix, use default name
   if (!suffixes.length || suffixes[0]) {
@@ -366,38 +374,38 @@ Zotero.DBConnection.prototype.getNextName = Zotero.Promise.coroutine(function* (
     num++;
   }
   return name + ' ' + num;
-});
+};
 
 
 //
 // Async methods
 //
 //
-// Zotero.DB.executeTransaction(function* (conn) {
-//     var created = yield Zotero.DB.queryAsync("CREATE TEMPORARY TABLE tmpFoo (foo TEXT, bar INT)");
+// Zotero.DB.executeTransaction(async function (conn) {
+//     var created = await Zotero.DB.queryAsync("CREATE TEMPORARY TABLE tmpFoo (foo TEXT, bar INT)");
 //     
 //     // created == true
 //     
-//     var result = yield Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('a', ?)", 1);
+//     var result = await Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('a', ?)", 1);
 //     
 //     // result == 1
 //     
-//     yield Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('b', 2)");
-//     yield Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('c', 3)");
-//     yield Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('d', 4)");
+//     await Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('b', 2)");
+//     await Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('c', 3)");
+//     await Zotero.DB.queryAsync("INSERT INTO tmpFoo VALUES ('d', 4)");
 //     
-//     var value = yield Zotero.DB.valueQueryAsync("SELECT foo FROM tmpFoo WHERE bar=?", 2);
+//     var value = await Zotero.DB.valueQueryAsync("SELECT foo FROM tmpFoo WHERE bar=?", 2);
 //     
 //     // value == "b"
 //     
-//     var vals = yield Zotero.DB.columnQueryAsync("SELECT foo FROM tmpFoo");
+//     var vals = await Zotero.DB.columnQueryAsync("SELECT foo FROM tmpFoo");
 //     
 //     // '0' => "a"
 //     // '1' => "b"
 //     // '2' => "c"
 //     // '3' => "d"
 //     
-//     let rows = yield Zotero.DB.queryAsync("SELECT * FROM tmpFoo");
+//     let rows = await Zotero.DB.queryAsync("SELECT * FROM tmpFoo");
 //     for (let i=0; i<rows.length; i++) {
 //         let row = rows[i];
 //         // row.foo == 'a', row.bar == 1
@@ -405,19 +413,16 @@ Zotero.DBConnection.prototype.getNextName = Zotero.Promise.coroutine(function* (
 //         // row.foo == 'c', row.bar == 3
 //         // row.foo == 'd', row.bar == 4
 //     }
-//     
-//    return rows;
-// })
-// .then(function (rows) {
-//     // rows == same as above
 // });
 //
 /**
- * @param {Function} func - Generator function that yields promises,
- *                          generally from queryAsync() and similar
+ * @param {Function} func - Async function containing `await Zotero.DB.queryAsync()` and similar
+ * @param {Object} [options]
+ * @param {Boolean} [options.disableForeignKeys] - Disable foreign key constraints before
+ *    transaction and re-enable after. (`PRAGMA foreign_keys=0|1` is a no-op during a transaction.)
  * @return {Promise} - Promise for result of generator function
  */
-Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(function* (func, options) {
+Zotero.DBConnection.prototype.executeTransaction = async function (func, options) {
   options = options || {};
   var resolve;
 
@@ -435,7 +440,7 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
 
   try {
     while (this._transactionID) {
-      yield this.waitForTransaction(id).timeout(options.waitTimeout || 30000);
+      await this.waitForTransaction(id).timeout(options.waitTimeout || 30000);
     }
     startedTransaction = true;
     this._transactionID = id;
@@ -455,8 +460,13 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
         this._callbacks.begin[i](id);
       }
     }
-    var conn = this._getConnection(options) || (yield this._getConnectionAsync(options));
-    var result = yield conn.executeTransaction(func);
+
+    if (options.disableForeignKeys) {
+      await this.queryAsync("PRAGMA foreign_keys = 0");
+    }
+
+    var conn = this._getConnection(options) || (await this._getConnectionAsync(options));
+    var result = await conn.executeTransaction(func);
     Zotero.debug(`Committed DB transaction ${id}`, 4);
 
     // Clear transaction time
@@ -466,7 +476,7 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
 
     if (options.vacuumOnCommit) {
       Zotero.debug('Vacuuming database');
-      yield this.queryAsync('VACUUM');
+      await this.queryAsync('VACUUM');
       Zotero.debug('Done vacuuming');
 
     }
@@ -483,13 +493,13 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
     // Run temporary commit callbacks
     var f;
     while (f = this._callbacks.current.commit.shift()) {
-      yield Zotero.Promise.resolve(f(id));
+      await Zotero.Promise.resolve(f(id));
     }
 
     // Run commit callbacks
     for (var i = 0; i < this._callbacks.commit.length; i++) {
       if (this._callbacks.commit[i]) {
-        yield this._callbacks.commit[i](id);
+        await this._callbacks.commit[i](id);
       }
     }
 
@@ -516,19 +526,23 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
     // Run temporary commit callbacks
     var f;
     while (f = this._callbacks.current.rollback.shift()) {
-      yield Zotero.Promise.resolve(f(id));
+      await Zotero.Promise.resolve(f(id));
     }
 
     // Run rollback callbacks
     for (var i = 0; i < this._callbacks.rollback.length; i++) {
       if (this._callbacks.rollback[i]) {
-        yield Zotero.Promise.resolve(this._callbacks.rollback[i](id));
+        await Zotero.Promise.resolve(this._callbacks.rollback[i](id));
       }
     }
 
     throw e;
   } finally
   {
+    if (options.disableForeignKeys) {
+      await this.queryAsync("PRAGMA foreign_keys = 1");
+    }
+
     // Reset options back to their previous values
     if (options) {
       for (let option in options) {
@@ -541,7 +555,7 @@ Zotero.DBConnection.prototype.executeTransaction = Zotero.Promise.coroutine(func
       resolve.call();
     }
   }
-});
+};
 
 
 Zotero.DBConnection.prototype.inTransaction = function () {
@@ -570,16 +584,16 @@ Zotero.DBConnection.prototype.requireTransaction = function () {
 
 
 /**
-    * @param {String} sql SQL statement to run
-    * @param {Array|String|Integer} [params] SQL parameters to bind
-    * @return {Promise|Array} A promise for an array of rows. The individual
-    *                         rows are Proxy objects that return values from the
-    *                         underlying mozIStorageRows based on column names.
-    */
-Zotero.DBConnection.prototype.queryAsync = Zotero.Promise.coroutine(function* (sql, params, options) {
+ * @param {String} sql SQL statement to run
+ * @param {Array|String|Integer} [params] SQL parameters to bind
+ * @return {Promise|Array} A promise for an array of rows. The individual
+ *                         rows are Proxy objects that return values from the
+ *                         underlying mozIStorageRows based on column names.
+ */
+Zotero.DBConnection.prototype.queryAsync = async function (sql, params, options) {
   try {
     let onRow = null;
-    let conn = this._getConnection(options) || (yield this._getConnectionAsync(options));
+    let conn = this._getConnection(options) || (await this._getConnectionAsync(options));
     if (!options || !options.noParseParams) {
       [sql, params] = this.parseQueryAndParams(sql, params);
     }
@@ -601,10 +615,10 @@ Zotero.DBConnection.prototype.queryAsync = Zotero.Promise.coroutine(function* (s
     }
     let rows;
     if (options && options.noCache) {
-      rows = yield conn.execute(sql, params, onRow);
+      rows = await conn.execute(sql, params, onRow);
     } else
     {
-      rows = yield conn.executeCached(sql, params, onRow);
+      rows = await conn.executeCached(sql, params, onRow);
     }
     if (failed) {
       throw failed;
@@ -658,22 +672,30 @@ Zotero.DBConnection.prototype.queryAsync = Zotero.Promise.coroutine(function* (s
     }
   }
   catch (e) {
+    await this._checkException(e);
+
     if (e.errors && e.errors[0]) {
       var eStr = e + "";
       eStr = eStr.indexOf("Error: ") == 0 ? eStr.substr(7) : e;
       throw new Error(eStr + ' [QUERY: ' + sql + '] ' + (
-      params ? '[PARAMS: ' + params.join(', ') + '] ' : '') +
+      params ?
+      '[PARAMS: ' + (
+      Array.isArray(params) ?
+      params.map((x) => JSON.stringify(x)).join(', ') :
+      JSON.stringify(params)) +
+      '] ' :
+      '') +
       '[ERROR: ' + e.errors[0].message + ']');
     } else
     {
       throw e;
     }
   }
-});
+};
 
 
 Zotero.DBConnection.prototype.queryTx = function (sql, params, options) {
-  return this.executeTransaction(function* () {
+  return this.executeTransaction(async function () {
     options = options || {};
     delete options.tx;
     return this.queryAsync(sql, params, options);
@@ -682,18 +704,24 @@ Zotero.DBConnection.prototype.queryTx = function (sql, params, options) {
 
 
 /**
-    * @param {String} sql  SQL statement to run
-    * @param {Array|String|Integer} [params]  SQL parameters to bind
-    * @return {Promise<Array|Boolean>}  A promise for either the value or FALSE if no result
-    */
-Zotero.DBConnection.prototype.valueQueryAsync = Zotero.Promise.coroutine(function* (sql, params, options = {}) {
+ * @param {String} sql  SQL statement to run
+ * @param {Array|String|Integer} [params]  SQL parameters to bind
+ * @return {Promise<Array|Boolean>}  A promise for either the value or FALSE if no result
+ */
+Zotero.DBConnection.prototype.valueQueryAsync = async function (sql, params, options = {}) {
   try {
-    let conn = this._getConnection(options) || (yield this._getConnectionAsync(options));
+    let conn = this._getConnection(options) || (await this._getConnectionAsync(options));
     [sql, params] = this.parseQueryAndParams(sql, params);
     if (Zotero.Debug.enabled) {
       this.logQuery(sql, params, options);
     }
-    let rows = yield conn.executeCached(sql, params);
+    let rows;
+    if (options && options.noCache) {
+      rows = await conn.execute(sql, params);
+    } else
+    {
+      rows = await conn.executeCached(sql, params);
+    }
     return rows.length ? rows[0].getResultByIndex(0) : false;
   }
   catch (e) {
@@ -708,35 +736,39 @@ Zotero.DBConnection.prototype.valueQueryAsync = Zotero.Promise.coroutine(functio
       throw e;
     }
   }
-});
-
-
-/**
-     * @param {String} sql SQL statement to run
-     * @param {Array|String|Integer} [params] SQL parameters to bind
-     * @return {Promise<Object>}  A promise for a proxied storage row
-     */
-Zotero.DBConnection.prototype.rowQueryAsync = function (sql, params) {
-  return this.queryAsync(sql, params).
-  then(function (rows) {
-    return rows.length ? rows[0] : false;
-  });
 };
 
 
 /**
-    * @param {String} sql SQL statement to run
-    * @param {Array|String|Integer} [params] SQL parameters to bind
-    * @return {Promise<Array>}  A promise for an array of values in the column
-    */
-Zotero.DBConnection.prototype.columnQueryAsync = Zotero.Promise.coroutine(function* (sql, params, options = {}) {
+ * @param {String} sql SQL statement to run
+ * @param {Array|String|Integer} [params] SQL parameters to bind
+ * @return {Promise<Object>}  A promise for a proxied storage row
+ */
+Zotero.DBConnection.prototype.rowQueryAsync = async function (sql, params) {
+  var rows = await this.queryAsync(sql, params);
+  return rows.length ? rows[0] : false;
+};
+
+
+/**
+ * @param {String} sql SQL statement to run
+ * @param {Array|String|Integer} [params] SQL parameters to bind
+ * @return {Promise<Array>}  A promise for an array of values in the column
+ */
+Zotero.DBConnection.prototype.columnQueryAsync = async function (sql, params, options = {}) {
   try {
-    let conn = this._getConnection(options) || (yield this._getConnectionAsync(options));
+    let conn = this._getConnection(options) || (await this._getConnectionAsync(options));
     [sql, params] = this.parseQueryAndParams(sql, params);
     if (Zotero.Debug.enabled) {
       this.logQuery(sql, params, options);
     }
-    let rows = yield conn.executeCached(sql, params);
+    let rows;
+    if (options && options.noCache) {
+      rows = await conn.execute(sql, params);
+    } else
+    {
+      rows = await conn.executeCached(sql, params);
+    }
     var column = [];
     for (let i = 0, len = rows.length; i < len; i++) {
       column.push(rows[i].getResultByIndex(0));
@@ -755,7 +787,7 @@ Zotero.DBConnection.prototype.columnQueryAsync = Zotero.Promise.coroutine(functi
       throw e;
     }
   }
-});
+};
 
 
 Zotero.DBConnection.prototype.logQuery = function (sql, params = [], options) {
@@ -779,21 +811,32 @@ Zotero.DBConnection.prototype.logQuery = function (sql, params = [], options) {
 };
 
 
-Zotero.DBConnection.prototype.tableExists = Zotero.Promise.coroutine(function* (table, db) {
-  yield this._getConnectionAsync();
+Zotero.DBConnection.prototype.tableExists = async function (table, db) {
+  await this._getConnectionAsync();
   var prefix = db ? db + '.' : '';
   var sql = `SELECT COUNT(*) FROM ${prefix}sqlite_master WHERE type='table' AND tbl_name=?`;
-  var count = yield this.valueQueryAsync(sql, [table]);
+  var count = await this.valueQueryAsync(sql, [table]);
   return !!count;
-});
+};
 
 
-/**
-     * Parse SQL string and execute transaction with all statements
-     *
-     * @return {Promise}
-     */
-Zotero.DBConnection.prototype.executeSQLFile = Zotero.Promise.coroutine(function* (sql) {
+Zotero.DBConnection.prototype.columnExists = async function (table, column) {
+  await this._getConnectionAsync();
+  var sql = `SELECT COUNT(*) FROM pragma_table_info(?) WHERE name=?`;
+  var count = await this.valueQueryAsync(sql, [table, column]);
+  return !!count;
+};
+
+
+Zotero.DBConnection.prototype.indexExists = async function (index, db) {
+  await this._getConnectionAsync();
+  var prefix = db ? db + '.' : '';
+  var sql = `SELECT COUNT(*) FROM ${prefix}sqlite_master WHERE type='index' AND name=?`;
+  return !!(await this.valueQueryAsync(sql, [index]));
+};
+
+
+Zotero.DBConnection.prototype.parseSQLFile = function (sql) {
   var nonCommentRE = /^[^-]/;
   var trailingCommentRE = /^(.*?)(?:--.+)?$/;
 
@@ -801,28 +844,38 @@ Zotero.DBConnection.prototype.executeSQLFile = Zotero.Promise.coroutine(function
   // Ugly hack to parse triggers with embedded semicolons
   .replace(/;---/g, "TEMPSEMI").
   split("\n").
-  filter(x => nonCommentRE.test(x)).
-  map(x => x.match(trailingCommentRE)[1]).
+  filter((x) => nonCommentRE.test(x)).
+  map((x) => x.match(trailingCommentRE)[1]).
   join("");
   if (sql.substr(-1) == ";") {
     sql = sql.substr(0, sql.length - 1);
   }
 
   var statements = sql.split(";").
-  map(x => x.replace(/TEMPSEMI/g, ";"));
+  map((x) => x.replace(/TEMPSEMI/g, ";"));
 
+  return statements;
+};
+
+
+/**
+ * Parse SQL string and execute transaction with all statements
+ *
+ * @return {Promise}
+ */
+Zotero.DBConnection.prototype.executeSQLFile = async function (sql) {
   this.requireTransaction();
-
+  var statements = this.parseSQLFile(sql);
   var statement;
   while (statement = statements.shift()) {
-    yield this.queryAsync(statement);
+    await this.queryAsync(statement, false, { noCache: true });
   }
-});
+};
 
 
 /*
-     * Implements nsIObserver
-     */
+ * Implements nsIObserver
+ */
 Zotero.DBConnection.prototype.observe = function (subject, topic, data) {
   switch (topic) {
     case 'idle':
@@ -832,85 +885,79 @@ Zotero.DBConnection.prototype.observe = function (subject, topic, data) {
 };
 
 
-// TEMP
-Zotero.DBConnection.prototype.vacuum = function () {
-  return this.executeTransaction(function* () {}, { vacuumOnCommit: true });
+Zotero.DBConnection.prototype.numCachedStatements = function () {
+  return this._connection._connectionData._cachedStatements.size;
+};
+
+
+Zotero.DBConnection.prototype.getCachedStatements = function () {
+  return [...this._connection._connectionData._cachedStatements].map((x) => x[0]);
 };
 
 
 // TEMP
-Zotero.DBConnection.prototype.info = Zotero.Promise.coroutine(function* () {
+Zotero.DBConnection.prototype.vacuum = function () {
+  return this.executeTransaction(async function () {}, { vacuumOnCommit: true });
+};
+
+
+// TEMP
+Zotero.DBConnection.prototype.info = async function () {
   var info = {};
   var pragmas = ['auto_vacuum', 'cache_size', 'main.locking_mode', 'page_size'];
   for (let p of pragmas) {
-    info[p] = yield Zotero.DB.valueQueryAsync(`PRAGMA ${p}`);
+    info[p] = await Zotero.DB.valueQueryAsync(`PRAGMA ${p}`);
   }
   return info;
-});
+};
 
 
-Zotero.DBConnection.prototype.integrityCheck = Zotero.Promise.coroutine(function* () {
-  var ok = yield this.valueQueryAsync("PRAGMA integrity_check");
+Zotero.DBConnection.prototype.quickCheck = async function () {
+  var ok = await this.valueQueryAsync("PRAGMA quick_check(1)");
   return ok == 'ok';
-});
+};
 
 
-Zotero.DBConnection.prototype.checkException = function (e) {
-  if (this._externalDB) {
-    return true;
-  }
+Zotero.DBConnection.prototype.integrityCheck = async function () {
+  var ok = await this.valueQueryAsync("PRAGMA integrity_check(1)");
+  return ok == 'ok';
+};
 
-  if (e.message.includes(this.DB_CORRUPTION_STRING)) {
-    // Write corrupt marker to data directory
-    var file = Zotero.File.pathToFile(this._dbPath + '.is.corrupt');
-    Zotero.File.putContents(file, '');
 
-    this._dbIsCorrupt = true;
-
-    var ps = Services.prompt;
-
-    var buttonFlags = ps.BUTTON_POS_0 * ps.BUTTON_TITLE_IS_STRING +
-    ps.BUTTON_POS_1 * ps.BUTTON_TITLE_IS_STRING;
-
-    var index = ps.confirmEx(null,
-    Zotero.getString('general.error'),
-    Zotero.getString('db.dbCorrupted', this._dbName) + '\n\n' + Zotero.getString('db.dbCorrupted.restart', Zotero.appName),
-    buttonFlags,
-    Zotero.getString('general.restartNow'),
-    Zotero.getString('general.restartLater'),
-    null, null, {});
-
-    if (index == 0) {
-      var appStartup = Services.startup;
-      appStartup.quit(Components.interfaces.nsIAppStartup.eRestart);
-      appStartup.quit(Components.interfaces.nsIAppStartup.eAttemptQuit);
-    }
-
-    Zotero.skipLoading = true;
-    return false;
-  }
-  return true;
+Zotero.DBConnection.prototype.isCorruptionError = function (e) {
+  return this.DB_CORRUPTION_STRINGS.some((x) => e.message.includes(x));
 };
 
 
 /**
-    * Close the database
-    * @param {Boolean} [permanent] If true, throw an error instead of
-    *     allowing code to re-open the database again
-    */
-Zotero.DBConnection.prototype.closeDatabase = Zotero.Promise.coroutine(function* (permanent) {
+ * Close the database
+ * @param {Boolean} [permanent] If true, throw an error instead of
+ *     allowing code to re-open the database again
+ */
+Zotero.DBConnection.prototype.closeDatabase = async function (permanent) {
   if (this._connection) {
+    // TODO: Replace with automatic detection of likely improperly cached statements
+    // (multiple similar statements, "tmp_", embedded ids)
+    if (Zotero.isSourceBuild) {
+      try {
+        Zotero.debug("Cached DB statements: " + this.numCachedStatements());
+      }
+      catch (e) {
+        Zotero.logError(e, 1);
+      }
+    }
+
     Zotero.debug("Closing database");
     this.closed = true;
-    yield this._connection.close();
+    await this._connection.close();
     this._connection = undefined;
     this._connection = permanent ? false : null;
     Zotero.debug("Database closed");
   }
-});
+};
 
 
-Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function* (suffix, force) {
+Zotero.DBConnection.prototype.backupDatabase = async function (suffix, force) {
   if (this.skipBackup || this._externalDB || Zotero.skipLoading) {
     this._debug("Skipping backup of database '" + this._dbName + "'", 1);
     return false;
@@ -941,7 +988,7 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
   // Start a promise that will be resolved when the backup is finished
   var resolveBackupPromise;
   if (this.inTransaction()) {
-    yield this.waitForTransaction();
+    await this.waitForTransaction();
   }
   this._backupPromise = new Zotero.Promise(function () {
     resolveBackupPromise = arguments[0];
@@ -960,9 +1007,9 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     // For standard backup, make sure last backup is old enough to replace
     if (!suffix && !force) {
       let backupFile = this._dbPath + '.bak';
-      if (yield OS.File.exists(backupFile)) {
-        let currentDBTime = (yield OS.File.stat(file)).lastModificationDate;
-        let lastBackupTime = (yield OS.File.stat(backupFile)).lastModificationDate;
+      if (await OS.File.exists(backupFile)) {
+        let currentDBTime = (await OS.File.stat(file)).lastModificationDate;
+        let lastBackupTime = (await OS.File.stat(backupFile)).lastModificationDate;
         if (currentDBTime == lastBackupTime) {
           Zotero.debug("Database '" + this._dbName + "' hasn't changed -- skipping backup");
           return;
@@ -984,9 +1031,9 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     // Copy via a temporary file so we don't run into disk space issues
     // after deleting the old backup file
     var tmpFile = this._dbPath + '.tmp';
-    if (yield OS.File.exists(tmpFile)) {
+    if (await OS.File.exists(tmpFile)) {
       try {
-        yield OS.File.remove(tmpFile);
+        await OS.File.remove(tmpFile);
       }
       catch (e) {
         if (e.name == 'NS_ERROR_FILE_ACCESS_DENIED') {
@@ -1000,7 +1047,7 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     // the lock is lost
     try {
       if (DB_LOCK_EXCLUSIVE) {
-        yield this.queryAsync("PRAGMA main.locking_mode=NORMAL", false, { inBackup: true });
+        await this.queryAsync("PRAGMA main.locking_mode=NORMAL", false, { inBackup: true });
       }
       storageService.backupDatabaseFile(
       Zotero.File.pathToFile(file),
@@ -1014,7 +1061,7 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     } finally
     {
       if (DB_LOCK_EXCLUSIVE) {
-        yield this.queryAsync("PRAGMA main.locking_mode=EXCLUSIVE", false, { inBackup: true });
+        await this.queryAsync("PRAGMA main.locking_mode=EXCLUSIVE", false, { inBackup: true });
       }
     }
 
@@ -1025,8 +1072,8 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     catch (e) {
       Zotero.logError(e);
       this._debug("Database file '" + OS.Path.basename(tmpFile) + "' can't be opened -- skipping backup");
-      if (yield OS.File.exists(tmpFile)) {
-        yield OS.File.remove(tmpFile);
+      if (await OS.File.exists(tmpFile)) {
+        await OS.File.remove(tmpFile);
       }
       return false;
     } finally
@@ -1038,7 +1085,7 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
             deferred.resolve();
           } });
 
-        yield deferred.promise;
+        await deferred.promise;
       }
     }
 
@@ -1046,8 +1093,8 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
     if (!suffix && numBackups > 1) {
       // Remove oldest backup file
       let targetFile = this._dbPath + '.' + (numBackups - 1) + '.bak';
-      if (yield OS.File.exists(targetFile)) {
-        yield OS.File.remove(targetFile);
+      if (await OS.File.exists(targetFile)) {
+        await OS.File.remove(targetFile);
       }
 
       // Shift old versions up
@@ -1058,24 +1105,24 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
         let targetFile = this._dbPath + '.' + targetNum + '.bak';
         let sourceFile = this._dbPath + '.' + (sourceNum ? sourceNum + '.bak' : 'bak');
 
-        if (!(yield OS.File.exists(sourceFile))) {
+        if (!(await OS.File.exists(sourceFile))) {
           continue;
         }
 
         Zotero.debug("Moving " + OS.Path.basename(sourceFile) +
         " to " + OS.Path.basename(targetFile));
-        yield OS.File.move(sourceFile, targetFile);
+        await OS.File.move(sourceFile, targetFile);
       }
     }
 
     let backupFile = this._dbPath + '.' + (suffix ? suffix + '.' : '') + 'bak';
 
     // Remove old backup file
-    if (yield OS.File.exists(backupFile)) {
+    if (await OS.File.exists(backupFile)) {
       OS.File.remove(backupFile);
     }
 
-    yield OS.File.move(tmpFile, backupFile);
+    await OS.File.move(tmpFile, backupFile);
     Zotero.debug("Backed up to " + OS.Path.basename(backupFile));
 
     return true;
@@ -1083,13 +1130,13 @@ Zotero.DBConnection.prototype.backupDatabase = Zotero.Promise.coroutine(function
   {
     resolveBackupPromise();
   }
-});
+};
 
 
 /**
-     * Escape '_', '%', and '\' in an SQL LIKE expression so that it can be used with ESCAPE '\' to
-     * prevent the wildcards from having special meaning
-     */
+ * Escape '_', '%', and '\' in an SQL LIKE expression so that it can be used with ESCAPE '\' to
+ * prevent the wildcards from having special meaning
+ */
 Zotero.DBConnection.prototype.escapeSQLExpression = function (expr) {
   return expr.replace(/([_%\\])/g, '\\$1');
 };
@@ -1112,8 +1159,8 @@ Zotero.DBConnection.prototype._getConnection = function (options) {
 };
 
 /*
-    * Retrieve a link to the data store asynchronously
-    */
+ * Retrieve a link to the data store asynchronously
+ */
 Zotero.DBConnection.prototype._getConnectionAsync = async function (options) {
   // If a backup is in progress, wait until it's done
   if (this._backupPromise && this._backupPromise.isPending() && (!options || !options.inBackup)) {
@@ -1135,13 +1182,11 @@ Zotero.DBConnection.prototype._getConnectionAsync = async function (options) {
   var store = Services.storage;
 
   var file = this._dbPath;
-  var backupFile = this._dbPath + '.bak';
-  var fileName = OS.Path.basename(file);
   var corruptMarker = this._dbPath + '.is.corrupt';
 
-  catchBlock: try {
+  try {
     if (await OS.File.exists(corruptMarker)) {
-      throw new Error(this.DB_CORRUPTION_STRING);
+      throw new Error(this.DB_CORRUPTION_STRINGS[0]);
     }
     this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
       path: file }));
@@ -1155,101 +1200,13 @@ Zotero.DBConnection.prototype._getConnectionAsync = async function (options) {
 
     Zotero.logError(e);
 
-    if (e.message.includes(this.DB_CORRUPTION_STRING)) {
-      this._debug(`Database file '${fileName}' corrupted`, 1);
-
-      // No backup file! Eek!
-      if (!(await OS.File.exists(backupFile))) {
-        this._debug("No backup file for DB '" + this._dbName + "' exists", 1);
-
-        // Save damaged filed
-        this._debug('Saving damaged DB file with .damaged extension', 1);
-        let damagedFile = this._dbPath + '.damaged';
-        await Zotero.File.moveToUnique(file, damagedFile);
-
-        // Create new main database
-        this._connection = store.openDatabase(file);
-
-        if (await OS.File.exists(corruptMarker)) {
-          await OS.File.remove(corruptMarker);
-        }
-
-        Zotero.alert(
-        null,
-        Zotero.getString('startupError'),
-        Zotero.getString('db.dbCorruptedNoBackup', fileName));
-
-        break catchBlock;
-      }
-
-      // Save damaged file
-      this._debug('Saving damaged DB file with .damaged extension', 1);
-      let damagedFile = this._dbPath + '.damaged';
-      await Zotero.File.moveToUnique(file, damagedFile);
-
-      // Test the backup file
-      try {
-        Zotero.debug("Asynchronously opening DB connection");
-        this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
-          path: backupFile }));
-
-      }
-      // Can't open backup either
-      catch (e) {
-        // Create new main database
-        this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
-          path: file }));
-
-
-        Zotero.alert(
-        null,
-        Zotero.getString('general.error'),
-        Zotero.getString('db.dbRestoreFailed', fileName));
-
-
-        if (await OS.File.exists(corruptMarker)) {
-          await OS.File.remove(corruptMarker);
-        }
-
-        break catchBlock;
-      }
-
-      this._connection = undefined;
-
-      // Copy backup file to main DB file
-      this._debug("Restoring database '" + this._dbName + "' from backup file", 1);
-      try {
-        await OS.File.copy(backupFile, file);
-      }
-      catch (e) {
-        // TODO: deal with low disk space
-        throw e;
-      }
-
-      // Open restored database
-      this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
-        path: file }));
-
-      this._debug('Database restored', 1);
-      Zotero.alert(
-      null,
-      Zotero.getString('general.warning'),
-      Zotero.getString('db.dbRestored', [
-      fileName,
-      Zotero.Date.getFileDateString(Zotero.File.pathToFile(backupFile)),
-      Zotero.Date.getFileTimeString(Zotero.File.pathToFile(backupFile))]));
-
-
-
-      if (await OS.File.exists(corruptMarker)) {
-        await OS.File.remove(corruptMarker);
-      }
-
-      break catchBlock;
+    if (this.DB_CORRUPTION_STRINGS.some((x) => e.message.includes(x))) {
+      await this._handleCorruptionMarker();
+    } else
+    {
+      // Some other error that we don't yet know how to deal with
+      throw e;
     }
-
-    // Some other error that we don't yet know how to deal with
-    throw e;
   }
 
   if (!this._externalDB) {
@@ -1281,6 +1238,185 @@ Zotero.DBConnection.prototype._getConnectionAsync = async function (options) {
 };
 
 
+Zotero.DBConnection.prototype._checkException = async function (e) {
+  if (this._externalDB || !this.isCorruptionError(e)) {
+    return true;
+  }
+
+  const supportURL = 'https://zotero.org/support/kb/corrupted_database';
+
+  var filename = OS.Path.basename(this._dbPath);
+  // Skip backups
+  this._dbIsCorrupt = true;
+
+  var backupDate = null;
+  var backupTime = null;
+  try {
+    let info = await OS.File.stat(this._dbPath + '.bak');
+    backupDate = info.lastModificationDate.toLocaleDateString();
+    backupTime = info.lastModificationDate.toLocaleTimeString();
+    Zotero.debug(`Found ${this._dbPath} with date of ${backupDate}`);
+  }
+  catch (e) {}
+
+  var ps = Services.prompt;
+  var buttonFlags = ps.BUTTON_POS_0 * ps.BUTTON_TITLE_IS_STRING +
+  ps.BUTTON_POS_1 * ps.BUTTON_TITLE_CANCEL;
+
+  var index = ps.confirmEx(null,
+  Zotero.getString('general.error'),
+  Zotero.getString('db.dbCorrupted', [Zotero.appName, filename]) + '\n\n' +
+  Zotero.getString('db.dbCorrupted.cloudStorage', Zotero.appName) + '\n\n' + (
+  backupDate ?
+  Zotero.getString(
+  'db.dbCorrupted.restoreFromLastAutomaticBackup',
+  [Zotero.appName, backupDate, backupTime]) +
+  '\n\n' +
+  Zotero.getString('db.dbCorrupted.viewMoreInformation', supportURL) :
+  Zotero.getString('db.dbCorrupted.repairOrRestore', Zotero.appName)),
+  buttonFlags,
+  backupDate ? Zotero.getString('db.dbCorrupted.automaticBackup') : Zotero.getString('general.moreInformation'),
+  null,
+  null,
+  null, {});
+
+  if (index == 0) {
+    // Write corrupt marker to data directory
+    let file = Zotero.File.pathToFile(this._dbPath + '.is.corrupt');
+    Zotero.File.putContents(file, '');
+    Zotero.skipLoading = true;
+    Zotero.Utilities.Internal.quit(true);
+  } else
+  if (index == 1) {
+  } else
+  {
+    Zotero.launchURL(supportURL);
+    Zotero.Utilities.Internal.quit();
+    Zotero.skipLoading = true;
+  }
+
+  return false;
+};
+
+
+/**
+ * @return {Boolean} - True if recovered, false if not
+ */
+Zotero.DBConnection.prototype._handleCorruptionMarker = async function () {
+  var storage = Services.storage;
+  var file = this._dbPath;
+  var fileName = OS.Path.basename(file);
+  var backupFile = this._dbPath + '.bak';
+  var corruptMarker = this._dbPath + '.is.corrupt';
+
+  this._debug(`Database file '${fileName}' corrupted`, 1);
+
+  // No backup file! Eek!
+  if (!(await OS.File.exists(backupFile))) {
+    this._debug("No backup file for DB '" + this._dbName + "' exists", 1);
+
+    // Save damaged filed
+    this._debug('Saving damaged DB file with .damaged extension', 1);
+    let damagedFile = this._dbPath + '.damaged';
+    damagedFile = await Zotero.File.moveToUnique(file, damagedFile);
+
+    // Create new main database
+    this._connection = storage.openDatabase(file);
+
+    if (await OS.File.exists(corruptMarker)) {
+      await OS.File.remove(corruptMarker);
+    }
+
+    Zotero.alert(
+    null,
+    Zotero.getString('startupError', Zotero.appName),
+    Zotero.getString(
+    'db.dbCorruptedNoBackup',
+    [Zotero.appName, fileName, OS.Path.basename(damagedFile)]));
+
+
+    return;
+  }
+
+  // Save damaged file
+  this._debug('Saving damaged DB file with .damaged extension', 1);
+  var damagedFile = this._dbPath + '.damaged';
+  damagedFile = await Zotero.File.moveToUnique(file, damagedFile);
+
+  // Test the backup file
+  try {
+    Zotero.debug("Asynchronously opening DB connection");
+    this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
+      path: backupFile }));
+
+    await this.closeDatabase();
+  }
+  // Can't open backup either
+  catch (e) {
+    // Create new main database
+    this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
+      path: file }));
+
+
+    Zotero.alert(
+    null,
+    Zotero.getString('general.error'),
+    Zotero.getString(
+    'db.dbRestoreFailed',
+    [Zotero.appName, fileName, OS.Path.basename(damagedFile)]));
+
+
+
+    if (await OS.File.exists(corruptMarker)) {
+      await OS.File.remove(corruptMarker);
+    }
+
+    return;
+  }
+
+  this._connection = undefined;
+
+  // Copy backup file to main DB file
+  this._debug("Restoring database '" + this._dbName + "' from backup file", 1);
+  try {
+    await OS.File.copy(backupFile, file);
+  }
+  catch (e) {
+    // TODO: deal with low disk space
+    throw e;
+  }
+
+  // Open restored database
+  this._connection = await Zotero.Promise.resolve(this.Sqlite.openConnection({
+    path: file }));
+
+  this._debug('Database restored', 1);
+  let backupDate = '';
+  let backupTime = '';
+  try {
+    let info = await OS.File.stat(backupFile);
+    backupDate = info.lastModificationDate.toLocaleDateString();
+    backupTime = info.lastModificationDate.toLocaleTimeString();
+  }
+  catch (e) {
+    Zotero.logError(e);
+  }
+  Zotero.alert(
+  null,
+  Zotero.getString('general.warning'),
+  Zotero.getString(
+  'db.dbRestored',
+  [Zotero.appName, fileName, backupDate, backupTime, OS.Path.basename(damagedFile)]) +
+  '\n\n' +
+  Zotero.getString('db.dbRestored.cloudStorage'));
+
+
+  if (await OS.File.exists(corruptMarker)) {
+    await OS.File.remove(corruptMarker);
+  }
+};
+
+
 Zotero.DBConnection.prototype._debug = function (str, level) {
   var prefix = this._dbName == 'zotero' ? '' : '[' + this._dbName + '] ';
   Zotero.debug(prefix + str, level);
