@@ -103,7 +103,7 @@ Zotero.Fulltext = Zotero.FullText = new function () {
       Zotero.addShutdownListener(this.unregisterContentProcessor.bind(this));
 
       // Start/stop content processor with full-text content syncing pref
-      Zotero.Prefs.registerObserver('sync.fulltext.enabled', enabled => {
+      Zotero.Prefs.registerObserver('sync.fulltext.enabled', (enabled) => {
         if (enabled) {
           this.registerContentProcessor();
         } else
@@ -219,9 +219,9 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-      * Returns true if MIME type is converted to text and cached before indexing
-      *   (e.g. application/pdf is run through pdftotext)
-      */
+   * Returns true if MIME type is converted to text and cached before indexing
+   *   (e.g. application/pdf is run through pdftotext)
+   */
   function isCachedMIMEType(mimeType) {
     switch (mimeType) {
       case 'application/pdf':
@@ -232,20 +232,20 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-     * Index multiple words at once
-     *
-     * @requireTransaction
-     * @param {Number} itemID
-     * @param {Array<string>} words
-     * @return {Promise}
-     */
+   * Index multiple words at once
+   *
+   * @requireTransaction
+   * @param {Number} itemID
+   * @param {Array<string>} words
+   * @return {Promise}
+   */
   var indexWords = Zotero.Promise.coroutine(function* (itemID, words, stats, version, synced) {
     Zotero.DB.requireTransaction();
     let chunk;
     yield Zotero.DB.queryAsync("DELETE FROM indexing.fulltextWords");
     while (words.length > 0) {
       chunk = words.splice(0, 100);
-      yield Zotero.DB.queryAsync('INSERT INTO indexing.fulltextWords (word) ' + chunk.map(x => 'SELECT ?').join(' UNION '), chunk);
+      yield Zotero.DB.queryAsync('INSERT INTO indexing.fulltextWords (word) ' + chunk.map((x) => 'SELECT ?').join(' UNION '), chunk);
     }
     yield Zotero.DB.queryAsync('INSERT OR IGNORE INTO fulltextWords (word) SELECT word FROM indexing.fulltextWords');
     yield Zotero.DB.queryAsync('DELETE FROM fulltextItemWords WHERE itemID = ?', [itemID]);
@@ -264,7 +264,7 @@ Zotero.Fulltext = Zotero.FullText = new function () {
       }
     }
     var sql = `REPLACE INTO fulltextItems (${cols.join(', ')}) ` +
-    `VALUES (${cols.map(_ => '?').join(', ')})`;
+    `VALUES (${cols.map((_) => '?').join(', ')})`;
     yield Zotero.DB.queryAsync(sql, params);
 
     yield Zotero.DB.queryAsync("DELETE FROM indexing.fulltextWords");
@@ -272,8 +272,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   var indexString = Zotero.Promise.coroutine(function* (text, charset, itemID, stats, version, synced) {
     var words = this.semanticSplitter(text, charset);
 
@@ -286,10 +286,11 @@ Zotero.Fulltext = Zotero.FullText = new function () {
       yield indexWords(itemID, words, stats, version, synced);
 
       /*
-                                                               var sql = "REPLACE INTO fulltextContent (itemID, textContent) VALUES (?,?)";
-                                                               Zotero.DB.query(sql, [itemID, {string:text}]);
-                                                               */
+      var sql = "REPLACE INTO fulltextContent (itemID, textContent) VALUES (?,?)";
+      Zotero.DB.query(sql, [itemID, {string:text}]);
+      */
 
+      Zotero.Notifier.queue('index', 'item', itemID);
       Zotero.Notifier.queue('refresh', 'item', itemID);
     }.bind(this));
 
@@ -303,10 +304,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-                  * @param {Document} document
-                  * @param {Number} itemID
-                  * @return {Promise}
-                  */
+   * @param {Document} document
+   * @param {Number} itemID
+   * @return {Promise}
+   */
   this.indexDocument = Zotero.Promise.coroutine(function* (document, itemID) {
     if (!itemID) {
       throw 'Item ID not provided to indexDocument()';
@@ -352,9 +353,9 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @param {String} path
-       * @param {Boolean} [complete=FALSE]  Index the file in its entirety, ignoring maxLength
-       */
+   * @param {String} path
+   * @param {Boolean} [complete=FALSE]  Index the file in its entirety, ignoring maxLength
+   */
   var indexFile = Zotero.Promise.coroutine(function* (path, contentType, charset, itemID, complete, stats) {
     if (!(yield OS.File.exists(path))) {
       Zotero.debug('File not found in indexFile()', 2);
@@ -418,14 +419,14 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-                  * Run PDF through pdfinfo and pdftotext to generate .zotero-ft-info
-                  * and .zotero-ft-cache, and pass the text file back to indexFile()
-                  *
-                  * @param {nsIFile} file
-                  * @param {Number} itemID
-                  * @param {Boolean} [allPages] - If true, index all pages rather than pdfMaxPages
-                  * @return {Promise}
-                  */
+   * Run PDF through pdfinfo and pdftotext to generate .zotero-ft-info
+   * and .zotero-ft-cache, and pass the text file back to indexFile()
+   *
+   * @param {nsIFile} file
+   * @param {Number} itemID
+   * @param {Boolean} [allPages] - If true, index all pages rather than pdfMaxPages
+   * @return {Promise}
+   */
   this.indexPDF = Zotero.Promise.coroutine(function* (filePath, itemID, allPages) {
     var maxPages = Zotero.Prefs.get('fulltext.pdfMaxPages');
     if (maxPages == 0) {
@@ -459,6 +460,7 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
     var { exec, args } = this.getPDFConverterExecAndArgs();
+    // Keep in sync with Item::attachmentText
     args.push('-nopgbrk');
 
     if (allPages) {
@@ -506,11 +508,11 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @param {Integer[]|Integer} items - One or more itemIDs
-       * @param {Object} [options]
-       * @param {Boolean} [options.complete=false] - Ignore page/character limits
-       * @param {Boolean} [options.ignoreErrors=false] - Continue on error instead of throwing
-       */
+   * @param {Integer[]|Integer} items - One or more itemIDs
+   * @param {Object} [options]
+   * @param {Boolean} [options.complete=false] - Ignore page/character limits
+   * @param {Boolean} [options.ignoreErrors=false] - Continue on error instead of throwing
+   */
   this.indexItems = async function (itemIDs, options = {}) {
     var complete;
     var ignoreErrors;
@@ -727,8 +729,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {String}  PHP-formatted POST data for items not yet downloaded
-       */
+   * @return {String}  PHP-formatted POST data for items not yet downloaded
+   */
   this.getUndownloadedPostData = Zotero.Promise.coroutine(function* () {
     // TODO: Redo for API syncing
 
@@ -765,19 +767,19 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * Save full-text content and stats to a cache file
-       *
-       * @param {Integer} libraryID
-       * @param {String} key - Item key
-       * @param {Object} data
-       * @param {String} data.content
-       * @param {Integer} [data.indexedChars]
-       * @param {Integer} [data.totalChars]
-       * @param {Integer} [data.indexedPages]
-       * @param {Integer} [data.totalPages]
-       * @param {Integer} version
-       * @return {Promise}
-       */
+   * Save full-text content and stats to a cache file
+   *
+   * @param {Integer} libraryID
+   * @param {String} key - Item key
+   * @param {Object} data
+   * @param {String} data.content
+   * @param {Integer} [data.indexedChars]
+   * @param {Integer} [data.totalChars]
+   * @param {Integer} [data.indexedPages]
+   * @param {Integer} [data.totalPages]
+   * @param {Integer} version
+   * @return {Promise}
+   */
   this.setItemContent = Zotero.Promise.coroutine(function* (libraryID, key, data, version) {
     var libraryKey = libraryID + "/" + key;
     var item = Zotero.Items.getByLibraryAndKey(libraryID, key);
@@ -828,19 +830,19 @@ Zotero.Fulltext = Zotero.FullText = new function () {
     }
     // If not yet indexed, add an empty row
     else {
-        yield Zotero.DB.queryAsync(
-        "REPLACE INTO fulltextItems (itemID, version, synced) VALUES (?, 0, ?)",
-        [itemID, synced]);
+      yield Zotero.DB.queryAsync(
+      "REPLACE INTO fulltextItems (itemID, version, synced) VALUES (?, 0, ?)",
+      [itemID, synced]);
 
-      }
+    }
 
     this.registerContentProcessor();
   });
 
 
   /**
-       * Start the idle observer for the background content processor
-       */
+   * Start the idle observer for the background content processor
+   */
   this.registerContentProcessor = function () {
     // Don't start idle observer during tests
     if (Zotero.test) return;
@@ -870,8 +872,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-      * Stop the idle observer and a running timer, if there is one
-      */
+   * Stop the idle observer and a running timer, if there is one
+   */
   this.stopContentProcessor = function () {
     Zotero.debug("Stopping full-text content processor");
     if (_processorTimeoutID) {
@@ -881,14 +883,14 @@ Zotero.Fulltext = Zotero.FullText = new function () {
   };
 
   /**
-      * Find items marked as having unprocessed cache files, run cache file processing on one item, and
-      * after a short delay call self again with the remaining items
-      *
-      * @param {Array<Integer>} itemIDs  An array of itemIDs to process; if this
-      *                                  is omitted, a database query is made
-      *                                  to find unprocessed content
-      * @return {Boolean}  TRUE if there's more content to process; FALSE otherwise
-      */
+   * Find items marked as having unprocessed cache files, run cache file processing on one item, and
+   * after a short delay call self again with the remaining items
+   *
+   * @param {Array<Integer>} itemIDs  An array of itemIDs to process; if this
+   *                                  is omitted, a database query is made
+   *                                  to find unprocessed content
+   * @return {Boolean}  TRUE if there's more content to process; FALSE otherwise
+   */
   this.processUnprocessedContent = Zotero.Promise.coroutine(function* (itemIDs) {
     // Idle observer can take a little while to trigger and may not cancel the setTimeout()
     // in time, so check idle time directly
@@ -946,16 +948,16 @@ Zotero.Fulltext = Zotero.FullText = new function () {
       }
       // When back from idle, stop the processor (but keep the idle observer registered)
       else if (topic == 'active') {
-          this.stopContentProcessor();
-        }
+        this.stopContentProcessor();
+      }
     }.bind(this) };
 
 
 
   /**
-                     * @param {Number} itemID
-                     * @return {Promise<Boolean>}
-                     */
+   * @param {Number} itemID
+   * @return {Promise<Boolean>}
+   */
   this.indexFromProcessorCache = Zotero.Promise.coroutine(function* (itemID) {
     try {
       var item = yield Zotero.Items.getAsync(itemID);
@@ -1066,22 +1068,22 @@ Zotero.Fulltext = Zotero.FullText = new function () {
   }
 
   /**
-     * Scan item files for a text string
-     *
-     * _items_ -- one or more attachment items to search
-     * _searchText_ -- text pattern to search for
-     * _mode_:
-     *    'phrase'
-     *    'regexp'
-     *    'regexpCS' -- case-sensitive regular expression
-     *
-     * Note:
-     *  - Slashes in regex are optional
-     *  - Add 'Binary' to the mode to search all files, not just text files
-     *
-     * @return {Promise<Array<Object>>} A promise for an array of match objects, with 'id' containing
-     *                                  an itemID and 'match' containing a string snippet
-     */
+   * Scan item files for a text string
+   *
+   * _items_ -- one or more attachment items to search
+   * _searchText_ -- text pattern to search for
+   * _mode_:
+   *    'phrase'
+   *    'regexp'
+   *    'regexpCS' -- case-sensitive regular expression
+   *
+   * Note:
+   *  - Slashes in regex are optional
+   *  - Add 'Binary' to the mode to search all files, not just text files
+   *
+   * @return {Promise<Array<Object>>} A promise for an array of match objects, with 'id' containing
+   *                                  an itemID and 'match' containing a string snippet
+   */
   this.findTextInItems = Zotero.Promise.coroutine(function* (items, searchText, mode) {
     if (!searchText) {
       return [];
@@ -1196,8 +1198,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-      * @requireTransaction
-      */
+   * @requireTransaction
+   */
   this.clearItemWords = Zotero.Promise.coroutine(function* (itemID, skipCacheClear) {
     Zotero.DB.requireTransaction();
 
@@ -1220,8 +1222,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   this.getPages = function (itemID) {
     var sql = "SELECT indexedPages, totalPages AS total " +
     "FROM fulltextItems WHERE itemID=?";
@@ -1230,11 +1232,11 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-      * Gets the number of pages from the PDF info cache file
-      *
-      * @private
-      * @return {Promise}
-      */
+   * Gets the number of pages from the PDF info cache file
+   *
+   * @private
+   * @return {Promise}
+   */
   var getTotalPagesFromFile = Zotero.Promise.coroutine(function* (itemID) {
     var file = OS.Path.join(
     Zotero.Attachments.getStorageDirectoryByID(itemID).path,
@@ -1257,8 +1259,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   function getChars(itemID) {
     var sql = "SELECT indexedChars, totalChars AS total " +
     "FROM fulltextItems WHERE itemID=?";
@@ -1267,10 +1269,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-     * Gets the number of characters from the PDF converter cache file
-     *
-     * @return {Promise}
-     */
+   * Gets the number of characters from the PDF converter cache file
+   *
+   * @return {Promise}
+   */
   var getTotalCharsFromFile = Zotero.Promise.coroutine(function* (itemID) {
     var item = yield Zotero.Items.getAsync(itemID);
     switch (item.attachmentContentType) {
@@ -1297,8 +1299,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   function setPages(itemID, obj) {
     var sql = "UPDATE fulltextItems SET indexedPages=?, totalPages=? WHERE itemID=?";
     return Zotero.DB.queryAsync(
@@ -1313,10 +1315,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-     * @param {Number} itemID
-     * @param {Object} obj
-     * @return {Promise}
-     */
+   * @param {Number} itemID
+   * @param {Object} obj
+   * @return {Promise}
+   */
   function setChars(itemID, obj) {
     var sql = "UPDATE fulltextItems SET indexedChars=?, totalChars=? WHERE itemID=?";
     return Zotero.DB.queryAsync(
@@ -1331,8 +1333,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-     * Gets the indexed state of an item, 
-     */
+   * Gets the indexed state of an item, 
+   */
   this.getIndexedState = Zotero.Promise.coroutine(function* (item) {
     if (!item.isAttachment()) {
       throw new Error('Item is not an attachment');
@@ -1401,8 +1403,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   this.getIndexStats = Zotero.Promise.coroutine(function* () {
     var sql = "SELECT COUNT(*) FROM fulltextItems WHERE synced != ? AND " +
     "((indexedPages IS NOT NULL AND indexedPages=totalPages) OR " +
@@ -1441,10 +1443,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-      * Returns true if an item can be reindexed
-      *
-      * Item must be a non-web-link attachment that isn't already fully indexed
-      */
+   * Returns true if an item can be reindexed
+   *
+   * Item must be a non-web-link attachment that isn't already fully indexed
+   */
   this.canReindex = Zotero.Promise.coroutine(function* (item) {
     if (item.isAttachment() &&
     item.attachmentLinkMode != Zotero.Attachments.LINK_MODE_LINKED_URL) {
@@ -1468,8 +1470,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * @return {Promise}
-       */
+   * @return {Promise}
+   */
   this.rebuildIndex = async function (unindexedOnly) {
     // Get all attachments other than web links
     var sql = "SELECT itemID FROM itemAttachments WHERE linkMode!=" +
@@ -1507,10 +1509,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-      * Clears full-text word index and all full-text cache files
-      *
-      * @return {Promise}
-      */
+   * Clears full-text word index and all full-text cache files
+   *
+   * @return {Promise}
+   */
   this.clearIndex = async function (skipLinkedURLs) {
     await Zotero.DB.executeTransaction(async function () {
       var sql = "DELETE FROM fulltextItems";
@@ -1542,8 +1544,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-      * Clears cache file for an item
-      */
+   * Clears cache file for an item
+   */
   var clearCacheFile = Zotero.Promise.coroutine(function* (itemID) {
     var item = yield Zotero.Items.getAsync(itemID);
     if (!item) {
@@ -1569,8 +1571,8 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-       * Clear cache files for all attachments
-       */
+   * Clear cache files for all attachments
+   */
   var clearCacheFiles = Zotero.Promise.coroutine(function* (skipLinkedURLs) {
     var sql = "SELECT itemID FROM itemAttachments";
     if (skipLinkedURLs) {
@@ -1584,15 +1586,15 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /*
-      function clearItemContent(itemID){
-      	Zotero.DB.query("DELETE FROM fulltextContent WHERE itemID=" + itemID);
-      }
-      */
+  function clearItemContent(itemID){
+  	Zotero.DB.query("DELETE FROM fulltextContent WHERE itemID=" + itemID);
+  }
+  */
 
 
   /**
-          * @return {Promise}
-          */
+   * @return {Promise}
+   */
   this.purgeUnusedWords = Zotero.Promise.coroutine(function* () {
     if (!Zotero.Prefs.get('purge.fulltext')) {
       return;
@@ -1607,10 +1609,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-       * Convert HTML to text for an item and cache the result
-       *
-       * @return {Promise}
-       */
+   * Convert HTML to text for an item and cache the result
+   *
+   * @return {Promise}
+   */
   var convertItemHTMLToText = Zotero.Promise.coroutine(function* (itemID, html, maxLength) {
     // Split elements to avoid word concatenation
     html = html.replace(/>/g, '> ');
@@ -1661,10 +1663,10 @@ Zotero.Fulltext = Zotero.FullText = new function () {
 
 
   /**
-     * @param {String} text
-     * @param {String} [charset]
-     * @return {Array<String>}
-     */
+   * @param {String} text
+   * @param {String} [charset]
+   * @return {Array<String>}
+   */
   this.semanticSplitter = function (text, charset) {
     if (!text) {
       Zotero.debug('No text to index');
