@@ -48,14 +48,6 @@ Zotero.Retractions = {
       return;
     }
 
-    // TEMP: Until we can figure out why some schema updates aren't going through despite the
-    // version number being incremented, create table here if it's missing
-    await Zotero.DB.queryAsync("CREATE TABLE IF NOT EXISTS retractedItems (\n	itemID INTEGER PRIMARY KEY,\n	data TEXT,\n	FOREIGN KEY (itemID) REFERENCES items(itemID) ON DELETE CASCADE\n);");
-    try {
-      await Zotero.DB.queryAsync("ALTER TABLE retractedItems ADD COLUMN flag INT DEFAULT 0");
-    }
-    catch (e) {}
-
     // Load mappings of keys (DOI hashes and PMIDs) to items and vice versa and register for
     // item changes so they can be kept up to date in notify().
     await this._cacheKeyMappings();
@@ -117,42 +109,42 @@ Zotero.Retractions = {
   },
 
   /**
-      * If item was retracted and the retraction hasn't been hidden
-      *
-      * @param {Zotero.Item} item
-      * @return {Boolean}
-      */
+   * If item was retracted and the retraction hasn't been hidden
+   *
+   * @param {Zotero.Item} item
+   * @return {Boolean}
+   */
   isRetracted: function (item) {
     var flag = this._retractedItems.get(item.id);
     return flag !== undefined && flag !== this.FLAG_HIDDEN;
   },
 
   /**
-      * If item was retracted and hasn't been marked to not show citation warnings
-      *
-      * @param {Zotero.Item}
-      * @return {Boolean}
-      */
+   * If item was retracted and hasn't been marked to not show citation warnings
+   *
+   * @param {Zotero.Item}
+   * @return {Boolean}
+   */
   shouldShowCitationWarning: function (item) {
     return this._retractedItems.get(item.id) === this.FLAG_NORMAL;
   },
 
   /**
-      * Don't show any future retraction warnings for this item
-      *
-      * @param {Zotero.Item} item
-      * @return {Promise}
-      */
+   * Don't show any future retraction warnings for this item
+   *
+   * @param {Zotero.Item} item
+   * @return {Promise}
+   */
   hideRetraction: async function (item) {
     return this._updateItemFlag(item, this.FLAG_HIDDEN);
   },
 
   /**
-      * Don't show future citation warnings for this item
-      *
-      * @param {Zotero.Item} item
-      * @return {Promise}
-      */
+   * Don't show future citation warnings for this item
+   *
+   * @param {Zotero.Item} item
+   * @return {Promise}
+   */
   disableCitationWarningsForItem: async function (item) {
     return this._updateItemFlag(item, this.FLAG_NO_CITATION_WARNING);
   },
@@ -189,8 +181,8 @@ Zotero.Retractions = {
       if (json.extra) {
         let { fields } = Zotero.Utilities.Internal.extractExtraFields(json.extra);
         let extraField = fields.get('DOI');
-        if (extraField && extraField.value) {
-          doi = extraField.value;
+        if (extraField) {
+          doi = extraField;
         }
       }
       if (doi) {
@@ -206,7 +198,7 @@ Zotero.Retractions = {
 
       // PMID
       if (json.extra) {
-        let pmid = this._extractPMID(json.extra);
+        let pmid = this._extractExtraFields(json.extra).pmid;
         if (pmid) {
           valuesToCheck.push({
             type: this.TYPE_PMID,
@@ -220,7 +212,7 @@ Zotero.Retractions = {
     // Check all possible values
     var keyIndexes = new Map();
     var prefixStringsToCheck = [];
-    for (let _ref of valuesToCheck) {let { type, value, index } = _ref;
+    for (let { type, value, index } of valuesToCheck) {
       // See if we've already cached a result for this key
       let key = this._valueToKey(type, value);
       let cachedResult = keyCache[type].get(key);
@@ -311,7 +303,7 @@ Zotero.Retractions = {
     // Changed
     && (previous != current
     // Explicitly hidden
-    || current && !Zotero.Utilities.Internal.getVirtualCollectionStateForLibrary(libraryID, 'retracted'))) {
+    || current && !Zotero.Prefs.getVirtualCollectionStateForLibrary(libraryID, 'retracted'))) {
       let promises = [];
       for (let zp of Zotero.getZoteroPanes()) {
         promises.push(zp.setVirtual(libraryID, 'retracted', current));
@@ -334,11 +326,11 @@ Zotero.Retractions = {
   },
 
   /**
-      * Return retraction data for an item
-      *
-      * @param {Zotero.Item} item
-      * @return {Object|false}
-      */
+   * Return retraction data for an item
+   *
+   * @param {Zotero.Item} item
+   * @return {Object|false}
+   */
   getData: async function (item) {
     var data = await Zotero.DB.valueQueryAsync(
     "SELECT data FROM retractedItems WHERE itemID=?", item.id);
@@ -419,10 +411,10 @@ Zotero.Retractions = {
           }
           // If a previous key value was cleared, re-map and re-check
           else if (this._itemKeys[typeID].get(item.id)) {
-              this._deleteItemKeyMappings(id);
-              this._updateItem(item);
-              continue;
-            }
+            this._deleteItemKeyMappings(id);
+            this._updateItem(item);
+            continue;
+          }
         }
         // We don't want to show the virtual collection for items in the trash, so add or
         // remove from the library set depending on whether it's in the trash. This is
@@ -448,8 +440,8 @@ Zotero.Retractions = {
   },
 
   /**
-      * Check for possible matches for items in the queue (debounced)
-      */
+   * Check for possible matches for items in the queue (debounced)
+   */
   checkQueuedItems: Zotero.Utilities.debounce(async function () {
     return this._checkQueuedItemsInternal();
   }, 1000),
@@ -518,7 +510,7 @@ Zotero.Retractions = {
       return;
     }
     var etag = req.getResponseHeader('ETag');
-    var list = req.response.split('\n').filter(x => x);
+    var list = req.response.split('\n').filter((x) => x);
 
     if (!list.length) {
       Zotero.logError("Empty retraction list from server");
@@ -543,9 +535,9 @@ Zotero.Retractions = {
     // Get all keys and compute prefixes to check for possible matches
     var prefixStrings = new Set([
     ...Array.from(this._keyItems[this.TYPE_DOI].keys()).
-    map(x => this.TYPE_DOI + x.substr(0, doiPrefixLength)),
+    map((x) => this.TYPE_DOI + x.substr(0, doiPrefixLength)),
     ...Array.from(this._keyItems[this.TYPE_PMID].keys()).
-    map(x => this.TYPE_PMID + x.substr(0, pmidPrefixLength))]);
+    map((x) => this.TYPE_PMID + x.substr(0, pmidPrefixLength))]);
 
     var prefixesToSend = new Set();
     for (let row of list) {
@@ -562,8 +554,6 @@ Zotero.Retractions = {
     }
 
     if (prefixesToSend.size) {
-      // TODO: Diff list and remove existing retractions that are missing
-
       let possibleMatches = await this._downloadPossibleMatches([...prefixesToSend]);
       await this._addPossibleMatches(possibleMatches, true);
     } else
@@ -577,9 +567,9 @@ Zotero.Retractions = {
 
 
   /**
-       * @param {String[]} prefixStrings
-       * @return {Object[]} - Results from API search
-       */
+   * @param {String[]} prefixStrings
+   * @return {Object[]} - Results from API search
+   */
   _downloadPossibleMatches: async function (prefixStrings) {
     var req = await Zotero.HTTP.request(
     "POST",
@@ -598,12 +588,12 @@ Zotero.Retractions = {
   },
 
   /**
-      * @param {Object[]} possibleMatches - Results from API search
-      * @param {Boolean} [removeExisting = false] - Remove retracted flag from all items that don't
-      *     match the results. This should only be true if possibleMatches includes all possible
-      *     matches in the database.
-      * @return {Number[]} - Array of added item ids
-      */
+   * @param {Object[]} possibleMatches - Results from API search
+   * @param {Boolean} [removeExisting = false] - Remove retracted flag from all items that don't
+   *     match the results. This should only be true if possibleMatches includes all possible
+   *     matches in the database.
+   * @return {Number[]} - Array of added item ids
+   */
   _addPossibleMatches: async function (possibleMatches, removeExisting) {
     // Look in the key mappings for local items that match and add them as retractions
     var addedItemIDs = new Set();
@@ -663,11 +653,11 @@ Zotero.Retractions = {
   _showAlert: async function (itemIDs) {
     // Don't show banner for items in the trash
     var items = await Zotero.Items.getAsync(itemIDs);
-    items = items.filter(item => !item.deleted);
+    items = items.filter((item) => !item.deleted);
     if (!items.length) {
       return;
     }
-    Zotero.Prefs.set('retractions.recentItems', JSON.stringify(items.map(item => item.id)));
+    Zotero.Prefs.set('retractions.recentItems', JSON.stringify(items.map((item) => item.id)));
     var zp = Zotero.getActiveZoteroPane();
     if (zp) {
       await zp.showRetractionBanner();
@@ -675,7 +665,7 @@ Zotero.Retractions = {
   },
 
   _getItemDOI: function (item) {
-    var itemDOI = item.getField('DOI') || item.getExtraField('doi');
+    var itemDOI = item.getField('DOI') || item.getExtraField('DOI');
     if (itemDOI) {
       itemDOI = Zotero.Utilities.cleanDOI(itemDOI);
     }
@@ -683,14 +673,17 @@ Zotero.Retractions = {
   },
 
   _getItemPMID: function (item) {
-    // TEMP
-    return this._extractPMID(item.getField('extra')) || null;
+    return this._extractExtraFields(item.getField('extra')).pmid;
   },
 
   // TEMP
-  _extractPMID: function (str) {
+  _extractExtraFields: function (str) {
+    var fields = {
+      doi: null,
+      pmid: null };
+
     if (!str) {
-      return false;
+      return fields;
     }
     var lines = str.split(/\n+/g);
     for (let line of lines) {
@@ -707,11 +700,18 @@ Zotero.Retractions = {
       .replace(/{:([^:]+):([^}]+)}/);
       value = value.trim();
 
-      if (field == 'pmid' || field == 'pubmedid') {
-        return value;
+      if (field == 'doi' && !fields.doi) {
+        fields.doi = value;
+      } else
+      if ((field == 'pmid' || field == 'pubmedid') && !fields.pmid) {
+        fields.pmid = value;
+      }
+
+      if (fields.doi && fields.pmid) {
+        break;
       }
     }
-    return false;
+    return fields;
   },
 
   _valueToKey: function (type, value) {
@@ -749,54 +749,61 @@ Zotero.Retractions = {
   },
 
   _cacheKeyMappings: async function () {
-    await this._cacheDOIMappings();
-    await this._cachePMIDMappings();
-  },
-
-  _cacheDOIMappings: async function () {
     this._keyItems[this.TYPE_DOI] = new Map();
     this._itemKeys[this.TYPE_DOI] = new Map();
-
-    var sql = "SELECT itemID AS id, value FROM itemData " +
-    "JOIN itemDataValues USING (valueID) WHERE fieldID=?";
-    var rows = await Zotero.DB.queryAsync(sql, Zotero.ItemFields.getID('DOI'));
-    for (let row of rows) {
-      let value = Zotero.Utilities.cleanDOI(row.value);
-      if (!value) continue;
-      this._addItemKeyMapping(this.TYPE_DOI, value, row.id);
-    }
-
-    // Extract from Extract field
-    sql = "SELECT itemID AS id, value FROM itemData " +
-    "JOIN itemDataValues USING (valueID) WHERE fieldID=?";
-    rows = await Zotero.DB.queryAsync(sql, Zotero.ItemFields.getID('extra'));
-    for (let row of rows) {
-      let { fields } = Zotero.Utilities.Internal.extractExtraFields(row.value);
-      let doi = fields.get('doi');
-      if (!doi || !doi.value) continue;
-      let value = Zotero.Utilities.cleanDOI(doi.value);
-      if (!value) continue;
-      this._addItemKeyMapping(this.TYPE_DOI, value, row.id);
-    }
-  },
-
-  _cachePMIDMappings: async function () {
     this._keyItems[this.TYPE_PMID] = new Map();
     this._itemKeys[this.TYPE_PMID] = new Map();
 
-    var sql = "SELECT itemID AS id, value FROM itemData " +
-    "JOIN itemDataValues USING (valueID) WHERE fieldID=?";
-    var rows = await Zotero.DB.queryAsync(sql, Zotero.ItemFields.getID('extra'));
+    var doiFieldID = Zotero.ItemFields.getID('DOI');
+    var extraFieldID = Zotero.ItemFields.getID('extra');
+
+    var sql = "SELECT itemID AS id, fieldID, value FROM itemData " +
+    "JOIN itemDataValues USING (valueID) WHERE fieldID IN (?, ?)";
+    var rows = await Zotero.DB.queryAsync(
+    sql,
+    [
+    doiFieldID,
+    extraFieldID]);
+
+
+
     for (let row of rows) {
-      /*
-                           let { fields } = Zotero.Utilities.Internal.extractExtraFields(row.value);
-                           let pmid = fields.get('pmid') || fields.get('pubmedID');
-                           if (!pmid || !pmid.value) continue;
-                           this._addItemKeyMapping(this.TYPE_PMID, pmid.value, row.id);
-                           */
-      let pmid = this._extractPMID(row.value);
-      if (!pmid) continue;
-      this._addItemKeyMapping(this.TYPE_PMID, pmid, row.id);
+      // DOI field
+      if (row.fieldID == doiFieldID) {
+        let value = Zotero.Utilities.cleanDOI(row.value);
+        if (value) {
+          this._addItemKeyMapping(this.TYPE_DOI, value, row.id);
+        }
+      }
+      // Extra field
+      else {
+        // DOI
+        /*
+        let { fields } = Zotero.Utilities.Internal.extractExtraFields(row.value);
+        let doi = fields.get('DOI');
+        if (!doi) continue;
+        */
+        let { doi, pmid } = this._extractExtraFields(row.value);
+
+        if (doi) {
+          let value = Zotero.Utilities.cleanDOI(doi);
+          if (value) {
+            this._addItemKeyMapping(this.TYPE_DOI, value, row.id);
+          }
+        }
+
+        // PMID
+        /*
+        let { fields } = Zotero.Utilities.Internal.extractExtraFields(row.value);
+        let pmid = fields.get('pmid') || fields.get('pubmedID');
+        if (!pmid) continue;
+        this._addItemKeyMapping(this.TYPE_PMID, pmid, row.id);
+        */
+        //let pmid = this._extractPMID(row.value);
+        if (pmid) {
+          this._addItemKeyMapping(this.TYPE_PMID, pmid, row.id);
+        }
+      }
     }
   },
 
@@ -825,9 +832,9 @@ Zotero.Retractions = {
   },
 
   /**
-      * Add new key mappings for an item, check if it matches a cached prefix, and queue it for full
-      * checking if so
-      */
+   * Add new key mappings for an item, check if it matches a cached prefix, and queue it for full
+   * checking if so
+   */
   _updateItem: function (item) {
     if (!item.isRegularItem()) {
       return;
@@ -919,7 +926,7 @@ Zotero.Retractions = {
     if (!(await OS.File.exists(cacheFile))) {
       return;
     }
-    var data = JSON.parse((await Zotero.File.getContentsAsync(cacheFile)));
+    var data = JSON.parse(await Zotero.File.getContentsAsync(cacheFile));
     if (data) {
       this._processCacheData(data);
     }
@@ -946,8 +953,8 @@ Zotero.Retractions = {
   },
 
   /**
-      * Cache prefix list in profile directory
-      */
+   * Cache prefix list in profile directory
+   */
   _saveCacheFile: async function (data, etag, doiPrefixLength, pmidPrefixLength) {
     var cacheFile = OS.Path.join(Zotero.Profile.dir, 'retractions.json');
     var cacheJSON = {
@@ -982,15 +989,15 @@ Zotero.Retractions = {
     }
     // Disable
     else {
-        if (this._notifierID) {
-          Zotero.Notifier.unregisterObserver(this._notifierID);
-          delete this._notifierID;
-        }
-        await this._removeAllEntries();
-        this._resetState();
-        let cacheFile = OS.Path.join(Zotero.Profile.dir, 'retractions.json');
-        await OS.File.remove(cacheFile);
+      if (this._notifierID) {
+        Zotero.Notifier.unregisterObserver(this._notifierID);
+        delete this._notifierID;
       }
+      await this._removeAllEntries();
+      this._resetState();
+      let cacheFile = OS.Path.join(Zotero.Profile.dir, 'retractions.json');
+      await OS.File.remove(cacheFile);
+    }
   },
 
   // https://retractionwatch.com/retraction-watch-database-user-guide/retraction-watch-database-user-guide-appendix-b-reasons/
