@@ -252,11 +252,11 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-      * Begin download process for individual file
-      *
-      * @param {Zotero.Sync.Storage.Request} request
-      * @return {Promise<Zotero.Sync.Storage.Result>}
-      */
+   * Begin download process for individual file
+   *
+   * @param {Zotero.Sync.Storage.Request} request
+   * @return {Promise<Zotero.Sync.Storage.Result>}
+   */
   downloadFile: Zotero.Promise.coroutine(function* (request) {
     var item = Zotero.Sync.Storage.Utilities.getItemFromRequest(request);
     if (!item) {
@@ -446,7 +446,10 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
             yield item.saveTx({ skipAll: true });
             // skipAll doesn't mark as unsynced, so do that separately
             yield item.updateSynced(false);
-            return new Zotero.Sync.Storage.Result();
+            return new Zotero.Sync.Storage.Result({
+              localChanges: true,
+              syncRequired: true });
+
           }
         }
 
@@ -463,9 +466,20 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
         if (smtime != mtime) {
           let shash = item.attachmentSyncedHash;
           if (shash && metadata.md5 && shash == metadata.md5) {
-            Zotero.debug("Last synced mod time for item " + item.libraryKey +
-            " doesn't match time on storage server but hash does -- ignoring");
-            return new Zotero.Sync.Storage.Result();
+            Zotero.debug(`Last synced mod time for item ${item.libraryKey} doesn't ` +
+            "match time on storage server but hash does -- using local file mtime");
+
+            yield this._setStorageFileMetadata(item);
+            item.attachmentSyncedModificationTime = fmtime;
+            item.attachmentSyncState = "in_sync";
+            yield item.saveTx({ skipAll: true });
+            // skipAll doesn't mark as unsynced, so do that separately
+            yield item.updateSynced(false);
+
+            return new Zotero.Sync.Storage.Result({
+              localChanges: true,
+              syncRequired: true });
+
           }
 
           Zotero.logError("Conflict -- last synced file mod time for item " +
@@ -494,10 +508,10 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
     }
 
     /*
-      updateSizeMultiplier(
-      	(100 - Zotero.Sync.Storage.compressionTracker.ratio) / 100
-      );
-      */
+    updateSizeMultiplier(
+    	(100 - Zotero.Sync.Storage.compressionTracker.ratio) / 100
+    );
+    */
 
     // Delete .prop file before uploading new .zip
     if (metadata) {
@@ -508,7 +522,7 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
         propURI,
         {
           successCodes: [200, 204, 404],
-          requestObserver: xmlhttp => request.setChannel(xmlhttp.channel),
+          requestObserver: (xmlhttp) => request.setChannel(xmlhttp.channel),
           errorDelayIntervals: this.ERROR_DELAY_INTERVALS,
           errorDelayMax: this.ERROR_DELAY_MAX,
           debug: true });
@@ -581,9 +595,9 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-       * @return {Promise}
-       * @throws {Zotero.Sync.Storage.Mode.WebDAV.VerificationError|Error}
-       */
+   * @return {Promise}
+   * @throws {Zotero.Sync.Storage.Mode.WebDAV.VerificationError|Error}
+   */
   checkServer: Zotero.Promise.coroutine(function* (options = {}) {
     // Clear URIs
     this._init();
@@ -715,8 +729,8 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
       // This can also be from IIS 6+, which is configured not to serve .prop files.
       // http://support.microsoft.com/kb/326965
       else if (req.status == 404) {
-          throw new this.VerificationError("FILE_MISSING_AFTER_UPLOAD", uri);
-        }
+        throw new this.VerificationError("FILE_MISSING_AFTER_UPLOAD", uri);
+      }
     } else
     if (req.status == 404) {
       // Include Authorization header from /zotero request,
@@ -749,10 +763,10 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-       * Handles the result of WebDAV verification, displaying an alert if necessary.
-       *
-       * @return bool True if the verification eventually succeeded, false otherwise
-       */
+   * Handles the result of WebDAV verification, displaying an alert if necessary.
+   *
+   * @return bool True if the verification eventually succeeded, false otherwise
+   */
   handleVerificationError: Zotero.Promise.coroutine(function* (err, window, skipSuccessMessage) {
     var promptService =
     Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
@@ -908,10 +922,10 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-       * Remove files on storage server that were deleted locally
-       *
-       * @param {Integer} libraryID
-       */
+   * Remove files on storage server that were deleted locally
+   *
+   * @param {Integer} libraryID
+   */
   purgeDeletedStorageFiles: Zotero.Promise.coroutine(function* (libraryID) {
     var d = new Date();
 
@@ -923,7 +937,7 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
     }
 
     // Add .zip extension
-    var files = files.map(file => file + ".zip");
+    var files = files.map((file) => file + ".zip");
 
     var results = yield this._deleteStorageFiles(files);
 
@@ -931,7 +945,7 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
     var toPurge = Zotero.Utilities.arrayUnique(
     results.deleted.concat(results.missing)
     // Strip file extension so we just have keys
-    .map(val => val.replace(/\.(prop|zip)$/, "")));
+    .map((val) => val.replace(/\.(prop|zip)$/, "")));
 
     if (toPurge.length > 0) {
       yield Zotero.Utilities.Internal.forEachChunkAsync(
@@ -955,8 +969,8 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-       * Delete orphaned storage files older than a week before last sync time
-       */
+   * Delete orphaned storage files older than a week before last sync time
+   */
   purgeOrphanedStorageFiles: Zotero.Promise.coroutine(function* () {
     var d = new Date();
     const libraryID = Zotero.Libraries.userLibraryID;
@@ -1011,8 +1025,8 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
       return Zotero.Utilities.xpath(this, path, { D: 'DAV:' });
     };
 
-    var syncQueueKeys = new Set((
-    yield Zotero.Sync.Data.Local.getObjectsFromSyncQueue('item', libraryID)));
+    var syncQueueKeys = new Set(
+    yield Zotero.Sync.Data.Local.getObjectsFromSyncQueue('item', libraryID));
 
     var deleteFiles = [];
     var trailingSlash = !!path.match(/\/$/);
@@ -1128,7 +1142,7 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
       {
         successCodes: [200, 300, 404],
         responseType: 'text',
-        requestObserver: xmlhttp => request.setChannel(xmlhttp.channel),
+        requestObserver: (xmlhttp) => request.setChannel(xmlhttp.channel),
         noCache: true,
         errorDelayIntervals: this.ERROR_DELAY_INTERVALS,
         errorDelayMax: this.ERROR_DELAY_MAX,
@@ -1218,10 +1232,10 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-       * Set mod time and hash of file on storage server
-       *
-       * @param	{Zotero.Item}	item
-       */
+   * Set mod time and hash of file on storage server
+   *
+   * @param	{Zotero.Item}	item
+   */
   _setStorageFileMetadata: Zotero.Promise.coroutine(function* (item) {
     var uri = this._getItemPropertyURI(item);
 
@@ -1307,8 +1321,8 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-      * Create a Zotero directory on the storage server
-      */
+   * Create a Zotero directory on the storage server
+   */
   _createServerDirectory: function () {
     return Zotero.HTTP.request(
     "MKCOL",
@@ -1323,35 +1337,35 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-      * Get the storage URI for an item
-      *
-      * @inner
-      * @param	{Zotero.Item}
-      * @return	{nsIURI}					URI of file on storage server
-      */
+   * Get the storage URI for an item
+   *
+   * @inner
+   * @param	{Zotero.Item}
+   * @return	{nsIURI}					URI of file on storage server
+   */
   _getItemURI: function (item) {
     return this.rootURI.mutate().setSpec(this.rootURI.spec + item.key + '.zip').finalize();
   },
 
 
   /**
-      * Get the storage property file URI for an item
-      *
-      * @inner
-      * @param	{Zotero.Item}
-      * @return	{nsIURI}					URI of property file on storage server
-      */
+   * Get the storage property file URI for an item
+   *
+   * @inner
+   * @param	{Zotero.Item}
+   * @return	{nsIURI}					URI of property file on storage server
+   */
   _getItemPropertyURI: function (item) {
     return this.rootURI.mutate().setSpec(this.rootURI.spec + item.key + '.prop').finalize();
   },
 
 
   /**
-      * Get the storage property file URI corresponding to a given item storage URI
-      *
-      * @param	{nsIURI}			Item storage URI
-      * @return	{nsIURI|FALSE}	Property file URI, or FALSE if not an item storage URI
-      */
+   * Get the storage property file URI corresponding to a given item storage URI
+   *
+   * @param	{nsIURI}			Item storage URI
+   * @return	{nsIURI|FALSE}	Property file URI, or FALSE if not an item storage URI
+   */
   _getPropertyURIFromItemURI: function (uri) {
     if (!uri.spec.match(/\.zip$/)) {
       return false;
@@ -1361,11 +1375,11 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
 
 
   /**
-      * @inner
-      * @param {String[]} files - Filenames of files to delete
-      * @return {Object} - Object with properties 'deleted', 'missing', and 'error', each
-      *     each containing filenames
-      */
+   * @inner
+   * @param {String[]} files - Filenames of files to delete
+   * @return {Object} - Object with properties 'deleted', 'missing', and 'error', each
+   *     each containing filenames
+   */
   _deleteStorageFiles: Zotero.Promise.coroutine(function* (files) {
     var results = {
       deleted: new Set(),
@@ -1423,17 +1437,19 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
             break;}
 
 
-        // If an item file URI, get the property URI
+        // If a .zip file URL, get the .prop file URI
         var deletePropURI = this._getPropertyURIFromItemURI(deleteURI);
-        // Only nsIURL has fileName
-        deletePropURI.QueryInterface(Ci.nsIURL);
-
-        // If we already deleted the prop file, skip it
-        if (!deletePropURI || results.deleted.has(deletePropURI.fileName)) {
+        // Not a .zip file URL
+        if (!deletePropURI) {
           return;
         }
-
+        // Only nsIURL has fileName
+        deletePropURI.QueryInterface(Ci.nsIURL);
         fileName = deletePropURI.fileName;
+        // Already deleted
+        if (results.deleted.has(fileName)) {
+          return;
+        }
 
         // Delete property file
         var req = yield Zotero.HTTP.request(
@@ -1463,8 +1479,8 @@ Zotero.Sync.Storage.Mode.WebDAV.prototype = {
     var caller = new ConcurrentCaller({
       numConcurrent: 4,
       stopOnError: true,
-      logger: msg => Zotero.debug(msg),
-      onError: e => Zotero.logError(e) });
+      logger: (msg) => Zotero.debug(msg),
+      onError: (e) => Zotero.logError(e) });
 
     yield caller.start(funcs);
 
