@@ -32,6 +32,7 @@ if (!Zotero.Sync) {
 // Initialized as Zotero.Sync.Runner in zotero.js
 Zotero.Sync.Runner_Module = function (options = {}) {
   const stopOnError = false;
+  const HTML_NS = 'http://www.w3.org/1999/xhtml';
 
   Zotero.defineProperty(this, 'enabled', {
     get: () => {
@@ -57,14 +58,14 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
   // Allows tests to set apiKey in options or as property, overriding login manager
   var _apiKey = options.apiKey;
-  Zotero.defineProperty(this, 'apiKey', { set: val => _apiKey = val });
+  Zotero.defineProperty(this, 'apiKey', { set: (val) => _apiKey = val });
 
   Components.utils.import("resource://zotero/concurrentCaller.js");
   this.caller = new ConcurrentCaller({
     numConcurrent: 4,
     stopOnError,
-    logger: msg => Zotero.debug(msg),
-    onError: e => Zotero.logError(e),
+    logger: (msg) => Zotero.debug(msg),
+    onError: (e) => Zotero.logError(e),
     Promise: Zotero.Promise });
 
 
@@ -74,6 +75,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
   var _delayPromises = [];
   var _firstInSession = true;
   var _syncInProgress = false;
+  var _queuedSyncOptions = [];
   var _stopping = false;
   var _canceller;
   var _manualSyncRequired = false; // TODO: make public?
@@ -84,7 +86,9 @@ Zotero.Sync.Runner_Module = function (options = {}) {
   var _lastSyncStatus;
   var _currentSyncStatusLabel;
   var _currentLastSyncLabel;
+  var _currentTooltipMessages;
   var _errors = [];
+  var _tooltipMessages = [];
 
   Zotero.addShutdownListener(() => this.stop());
 
@@ -92,6 +96,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
     return new Zotero.Sync.APIClient({
       baseURL: this.baseURL,
       apiVersion: this.apiVersion,
+      schemaVersion: this.globalSchemaVersion,
       apiKey: options.apiKey,
       caller: this.caller,
       cancellerReceiver: _cancellerReceiver });
@@ -100,16 +105,16 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Begin a sync session
-      *
-      * @param {Object}    [options]
-      * @param {Boolean}   [options.background=false]  Whether this is a background request, which
-      *                                                prevents some alerts from being shown
-      * @param {Integer[]} [options.libraries]         IDs of libraries to sync; skipped libraries must
-      *     be removed if unwanted
-      * @param {Function}  [options.onError]           Function to pass errors to instead of
-      *                                                handling internally (used for testing)
-      */
+   * Begin a sync session
+   *
+   * @param {Object}    [options]
+   * @param {Boolean}   [options.background=false]  Whether this is a background request, which
+   *                                                prevents some alerts from being shown
+   * @param {Integer[]} [options.libraries]         IDs of libraries to sync; skipped libraries must
+   *     be removed if unwanted
+   * @param {Function}  [options.onError]           Function to pass errors to instead of
+   *                                                handling internally (used for testing)
+   */
   this.sync = Zotero.serial(function (options = {}) {
     return this._sync(options);
   });
@@ -118,6 +123,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
   this._sync = Zotero.Promise.coroutine(function* (options) {
     // Clear message list
     _errors = [];
+    _tooltipMessages = [];
 
     // Shouldn't be possible because of serial()
     if (_syncInProgress) {
@@ -154,12 +160,12 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
       // If paused, wait until we're done
       while (true) {
-        if (_delayPromises.some(p => p.isPending())) {
+        if (_delayPromises.some((p) => p.isPending())) {
           this.setSyncStatus(Zotero.getString('sync.status.waiting'));
           Zotero.debug("Syncing is paused -- waiting to sync");
           yield Zotero.Promise.all(_delayPromises);
           // If more were added, continue
-          if (_delayPromises.some(p => p.isPending())) {
+          if (_delayPromises.some((p) => p.isPending())) {
             continue;
           }
           _delayPromises = [];
@@ -197,7 +203,13 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       let wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
       getService(Components.interfaces.nsIWindowMediator);
       let lastWin = wm.getMostRecentWindow("navigator:browser");
-      if (!(yield Zotero.Sync.Data.Local.checkUser(lastWin, keyInfo.userID, keyInfo.username))) {
+      let ok = yield Zotero.Sync.Data.Local.checkUser(
+      lastWin,
+      keyInfo.userID,
+      keyInfo.username,
+      keyInfo.displayName);
+
+      if (!ok) {
         Zotero.debug("User cancelled sync on username mismatch");
         return false;
       }
@@ -233,6 +245,19 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       options.libraries ? Array.from(options.libraries) : []);
 
 
+      // If file and full-text libraries are specified, limit to libraries we're already
+      // syncing
+      var fileLibrariesToSync = new Set(
+      options.fileLibraries ?
+      options.fileLibraries.filter((id) => librariesToSync.includes(id)) :
+      librariesToSync);
+
+      var fullTextLibrariesToSync = new Set(
+      options.fullTextLibraries ?
+      options.fullTextLibraries.filter((id) => librariesToSync.includes(id)) :
+      librariesToSync);
+
+
       _stopCheck();
 
       // If items not yet loaded for libraries we need, load them now
@@ -257,14 +282,17 @@ Zotero.Sync.Runner_Module = function (options = {}) {
         }
         let nextLibraries = yield _doDataSync(librariesToSync, engineOptions);
         // Remove failed libraries from the successful set
-        Zotero.Utilities.arrayDiff(librariesToSync, nextLibraries).forEach(libraryID => {
+        Zotero.Utilities.arrayDiff(librariesToSync, nextLibraries).forEach((libraryID) => {
           successfulLibraries.delete(libraryID);
         });
 
         _stopCheck();
 
-        // Run file sync on all libraries that passed the last data sync
-        librariesToSync = yield _doFileSync(nextLibraries, engineOptions);
+        // Run file sync on all allowed libraries that passed the last data sync
+        librariesToSync = yield _doFileSync(
+        nextLibraries.filter((libraryID) => fileLibrariesToSync.has(libraryID)),
+        engineOptions);
+
         if (librariesToSync.length) {
           attempt++;
           continue;
@@ -272,8 +300,11 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
         _stopCheck();
 
-        // Run full-text sync on all libraries that haven't failed a data sync
-        librariesToSync = yield _doFullTextSync([...successfulLibraries], engineOptions);
+        // Run full-text sync on all allowed libraries that haven't failed a data sync
+        librariesToSync = yield _doFullTextSync(
+        [...successfulLibraries].filter((libraryID) => fullTextLibrariesToSync.has(libraryID)),
+        engineOptions);
+
         if (librariesToSync.length) {
           attempt++;
           continue;
@@ -309,6 +340,12 @@ Zotero.Sync.Runner_Module = function (options = {}) {
         yield this._sync(options);
         return;
       }
+      // If an auto-sync was queued while a sync was ongoing, start again with its options
+      else if (_queuedSyncOptions.length) {
+        Zotero.debug("Restarting sync");
+        yield this._sync(JSON.parse(_queuedSyncOptions.shift()));
+        return;
+      }
 
       Zotero.debug("Done syncing");
       Zotero.Notifier.trigger('finish', 'sync', librariesToSync || []);
@@ -317,8 +354,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-       * Check key for current user info and return access info
-       */
+   * Check key for current user info and return access info
+   */
   this.checkAccess = Zotero.Promise.coroutine(function* (client, options = {}) {
     var json = yield client.getKeyInfo(options);
     Zotero.debug(json);
@@ -348,25 +385,34 @@ Zotero.Sync.Runner_Module = function (options = {}) {
         let index = ps.confirmEx(
         null,
         Zotero.getString('general.warning'),
-        Zotero.getString('account.warning.emptyLibrary', [keyInfo.username, Zotero.clientName]) + "\n\n" +
-        Zotero.getString('account.warning.existingDataElsewhere', Zotero.clientName),
+        Zotero.getString(
+        'account.warning.emptyLibrary',
+        [Zotero.clientName, OS.Path.basename(Zotero.DB.path)]) +
+        "\n\n" +
+        Zotero.getString(
+        'account.warning.emptyLibrary.dataWillBeDownloaded',
+        keyInfo.username) +
+
+        "\n\n" +
+        Zotero.getString(
+        'account.warning.existingDataElsewhere',
+        Zotero.clientName) +
+
+        "\n\n" +
+        Zotero.getString('dataDir.location', Zotero.DataDirectory.dir),
         ps.BUTTON_POS_0 * ps.BUTTON_TITLE_IS_STRING +
         ps.BUTTON_POS_1 * ps.BUTTON_TITLE_CANCEL +
         ps.BUTTON_POS_2 * ps.BUTTON_TITLE_IS_STRING,
         Zotero.getString('sync.sync'),
         null,
-        Zotero.getString('dataDir.changeDataDirectory'),
+        Zotero.getString('general.moreInformation'),
         null, {});
 
         if (index == 1) {
           return false;
         } else
         if (index == 2) {
-          var win = Services.wm.getMostRecentWindow("navigator:browser");
-          win.openDialog("chrome://zotero/content/preferences/preferences.xul", null, null, {
-            pane: 'zotero-prefpane-advanced',
-            tabIndex: 1 });
-
+          Zotero.launchURL('https://www.zotero.org/support/zotero_data#locating_missing_zotero_data');
           return false;
         }
       }
@@ -376,8 +422,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-       * @return {Promise<Integer[]> - IDs of libraries to sync
-       */
+   * @return {Promise<Integer[]> - IDs of libraries to sync
+   */
   this.checkLibraries = Zotero.Promise.coroutine(function* (client, options, keyInfo, libraries = []) {
     var access = keyInfo.access;
 
@@ -427,14 +473,14 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       }
 
       let remoteGroupVersions = yield client.getGroupVersions(keyInfo.userID);
-      let remoteGroupIDs = Object.keys(remoteGroupVersions).map(id => parseInt(id));
+      let remoteGroupIDs = Object.keys(remoteGroupVersions).map((id) => parseInt(id));
       let skippedGroups = Zotero.Sync.Data.Local.getSkippedGroups();
 
       // Remove skipped groups
       if (syncAllLibraries) {
         let newGroups = Zotero.Utilities.arrayDiff(remoteGroupIDs, skippedGroups);
         Zotero.Utilities.arrayDiff(remoteGroupIDs, newGroups).
-        forEach(id => {delete remoteGroupVersions[id];});
+        forEach((id) => {delete remoteGroupVersions[id];});
         remoteGroupIDs = newGroups;
       }
 
@@ -451,9 +497,9 @@ Zotero.Sync.Runner_Module = function (options = {}) {
           }
           // If not outdated, just add to library list
           else {
-              Zotero.debug(`Adding group library ${group.libraryID} to sync`);
-              libraries.push(group.libraryID);
-            }
+            Zotero.debug(`Adding group library ${group.libraryID} to sync`);
+            libraries.push(group.libraryID);
+          }
         } else
         {
           // If specific libraries were provided, ignore remote groups that don't
@@ -474,24 +520,24 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       let localGroups;
       if (syncAllLibraries) {
         localGroups = Zotero.Groups.getAll().
-        map(g => g.id)
+        map((g) => g.id)
         // Don't include skipped groups
-        .filter(id => skippedGroups.indexOf(id) == -1);
+        .filter((id) => skippedGroups.indexOf(id) == -1);
       } else
       {
         localGroups = libraries.
-        filter(id => Zotero.Libraries.get(id).libraryType == 'group').
-        map(id => Zotero.Groups.getGroupIDFromLibraryID(id));
+        filter((id) => Zotero.Libraries.get(id).libraryType == 'group').
+        map((id) => Zotero.Groups.getGroupIDFromLibraryID(id));
       }
       Zotero.debug("Local groups:");
       Zotero.debug(localGroups);
       remotelyMissingGroups = Zotero.Utilities.arrayDiff(localGroups, remoteGroupIDs).
-      map(id => Zotero.Groups.get(id));
+      map((id) => Zotero.Groups.get(id));
     }
     // No group access
     else {
-        remotelyMissingGroups = Zotero.Groups.getAll();
-      }
+      remotelyMissingGroups = Zotero.Groups.getAll();
+    }
 
     if (remotelyMissingGroups.length) {
       // TODO: What about explicit deletions?
@@ -512,7 +558,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       for (let group of remotelyMissingGroups) {
         // Ignore remotely missing archived groups
         if (group.archived) {
-          groupsToDownload = groupsToDownload.filter(groupID => groupID != group.id);
+          groupsToDownload = groupsToDownload.filter((groupID) => groupID != group.id);
           continue;
         }
 
@@ -523,8 +569,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
         }
         // If not all-groups access, key might just not have access
         else {
-            msg = "You no longer have access to the group \u2018" + group.name + "\u2019.";
-          }
+          msg = "You no longer have access to the group \u2018" + group.name + "\u2019.";
+        }
 
         msg += "\n\n" + "Would you like to remove it from this computer or keep it " +
         "as a read-only library?";
@@ -615,13 +661,13 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-       * Run sync engine for passed libraries
-       *
-       * @param {Integer[]} libraries
-       * @param {Object} options
-       * @param {Boolean} skipUpdateLastSyncTime
-       * @return {Integer[]} - Array of libraryIDs that completed successfully
-       */
+   * Run sync engine for passed libraries
+   *
+   * @param {Integer[]} libraries
+   * @param {Object} options
+   * @param {Boolean} skipUpdateLastSyncTime
+   * @return {Integer[]} - Array of libraryIDs that completed successfully
+   */
   var _doDataSync = Zotero.Promise.coroutine(function* (libraries, options, skipUpdateLastSyncTime) {
     var successfulLibraries = [];
     for (let libraryID of libraries) {
@@ -667,8 +713,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-                  * @return {Integer[]} - Array of libraries that need data syncing again
-                  */
+   * @return {Integer[]} - Array of libraries that need data syncing again
+   */
   var _doFileSync = Zotero.Promise.coroutine(function* (libraries, options) {
     Zotero.debug("Starting file syncing");
     var resyncLibraries = [];
@@ -745,8 +791,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-                  * @return {Integer[]} - Array of libraries that need data syncing again
-                  */
+   * @return {Integer[]} - Array of libraries that need data syncing again
+   */
   var _doFullTextSync = Zotero.Promise.coroutine(function* (libraries, options) {
     if (!Zotero.Prefs.get("sync.fulltext.enabled")) return [];
 
@@ -792,9 +838,9 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-                  * Get a storage controller for a given mode ('zfs', 'webdav'),
-                  * caching it if necessary
-                  */
+   * Get a storage controller for a given mode ('zfs', 'webdav'),
+   * caching it if necessary
+   */
   this.getStorageController = function (mode, options) {
     if (_storageControllers[mode]) {
       return _storageControllers[mode];
@@ -811,8 +857,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Download a single file on demand (not within a sync process)
-      */
+   * Download a single file on demand (not within a sync process)
+   */
   this.downloadFile = Zotero.Promise.coroutine(function* (item, requestCallbacks) {
     if (Zotero.HTTP.browserIsOffline()) {
       Zotero.debug("Browser is offline", 2);
@@ -840,8 +886,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       return false;
     }
 
-    if (!item.isImportedAttachment()) {
-      throw new Error("Not an imported attachment");
+    if (!item.isStoredFileAttachment()) {
+      throw new Error("Not a stored file attachment");
     }
 
     if (yield item.getFilePathAsync()) {
@@ -892,10 +938,10 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-       * @param {Integer} timeout - Timeout in seconds
-       * @param {Boolean} [recurring=false]
-       * @param {Object} [options] - Sync options
-       */
+   * @param {Integer} timeout - Timeout in seconds
+   * @param {Boolean} [recurring=false]
+   * @param {Object} [options] - Sync options (e.g., 'libraries', 'fileLibraries', 'fullTextLibraries')
+   */
   this.setSyncTimeout = function (timeout, recurring, options = {}) {
     if (!Zotero.Prefs.get('sync.autoSync') || !this.enabled) {
       return;
@@ -905,6 +951,10 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       throw new Error("Timeout not provided");
     }
 
+    if (timeout != parseInt(timeout)) {
+      throw new Error(`Timeout must be an integer (${timeout} given)`);
+    }
+
     if (_autoSyncTimer) {
       Zotero.debug("Cancelling auto-sync timer");
       _autoSyncTimer.cancel();
@@ -937,11 +987,13 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
         if (Zotero.locked) {
           Zotero.debug('Zotero is locked -- skipping auto-sync', 4);
+          _queueSyncOptions(mergedOpts);
           return;
         }
 
         if (_syncInProgress) {
           Zotero.debug('Sync already in progress -- skipping auto-sync', 4);
+          _queueSyncOptions(mergedOpts);
           return;
         }
 
@@ -963,6 +1015,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
     {
       if (_syncInProgress) {
         Zotero.debug('Sync in progress -- not setting auto-sync timeout', 4);
+        _queueSyncOptions(mergedOpts);
         return;
       }
 
@@ -974,6 +1027,18 @@ Zotero.Sync.Runner_Module = function (options = {}) {
   };
 
 
+  function _queueSyncOptions(options) {
+    var jsonOptions = JSON.stringify(options);
+    // Don't queue options if already queued
+    if (_queuedSyncOptions.includes(jsonOptions)) {
+      return;
+    }
+    Zotero.debug("Queueing sync options");
+    Zotero.debug(options);
+    _queuedSyncOptions.push(jsonOptions);
+  }
+
+
   this.clearSyncTimeout = function () {
     if (_autoSyncTimer) {
       _autoSyncTimer.cancel();
@@ -987,10 +1052,10 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Delay syncs until the returned function is called
-      *
-      * @return {Function} - Resolve function
-      */
+   * Delay syncs until the returned function is called
+   *
+   * @return {Function} - Resolve function
+   */
   this.delayIndefinite = function () {
     var resolve;
     var promise = new Zotero.Promise(function () {
@@ -1002,9 +1067,9 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Trigger updating of the main sync icon, the sync error icon, and
-      * library-specific sync error icons across all windows
-      */
+   * Trigger updating of the main sync icon, the sync error icon, and
+   * library-specific sync error icons across all windows
+   */
   this.addError = function (e, libraryID) {
     if (e.added) return;
     e.added = true;
@@ -1017,13 +1082,13 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   this.getErrorsByLibrary = function (libraryID) {
-    return _errors.filter(e => e.libraryID === libraryID);
+    return _errors.filter((e) => e.libraryID === libraryID);
   };
 
 
   /**
-      * Get most severe error type from an array of parsed errors
-      */
+   * Get most severe error type from an array of parsed errors
+   */
   this.getPrimaryErrorType = function (errors) {
     // Set highest priority error as the primary (sync error icon)
     var errorTypes = {
@@ -1033,7 +1098,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       upgrade: 4,
 
       // Skip these
-      animate: -1 };
+      animate: -1,
+      ignore: -2 };
 
     var state = false;
     for (let i = 0; i < errors.length; i++) {
@@ -1151,7 +1217,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
               // Open long tag fixer for every long tag in every editable library we're syncing
               var editableLibraries = options.libraries.
-              filter(x => Zotero.Libraries.get(x).editable);
+              filter((x) => Zotero.Libraries.get(x).editable);
               for (let libraryID of editableLibraries) {
                 let oldTagIDs = yield Zotero.Tags.getLongTagsInLibrary(libraryID);
                 for (let oldTagID of oldTagIDs) {
@@ -1227,21 +1293,21 @@ Zotero.Sync.Runner_Module = function (options = {}) {
             }
             // Field or creator too long
             else if (data && data.field) {
-                e.message = (data.field == 'creator' ?
-                Zotero.getString(
-                'sync.error.creatorTooLong',
-                [data.value]) :
+              e.message = (data.field == 'creator' ?
+              Zotero.getString(
+              'sync.error.creatorTooLong',
+              [data.value]) :
 
-                Zotero.getString(
-                'sync.error.fieldTooLong',
-                [data.field, data.value])) +
+              Zotero.getString(
+              'sync.error.fieldTooLong',
+              [data.field, data.value])) +
 
-                '\n\n' +
-                Zotero.getString(
-                'sync.error.reportSiteIssuesToForums',
-                Zotero.clientName);
+              '\n\n' +
+              Zotero.getString(
+              'sync.error.reportSiteIssuesToForums',
+              Zotero.clientName);
 
-              }
+            }
 
             // Include "Show Item in Library" button
             e.dialogButtonText = Zotero.getString('pane.items.showItemInLibrary');
@@ -1265,31 +1331,46 @@ Zotero.Sync.Runner_Module = function (options = {}) {
     }
     // Show warning for unknown data that couldn't be saved
     else if (e.name && e.name == 'ZoteroInvalidDataError') {
-        e.message = Zotero.getString(
-        'sync.error.invalidDataError',
-        [
-        Zotero.Libraries.get(e.libraryID).name,
-        Zotero.clientName]) +
+      let library = Zotero.Libraries.get(e.libraryID);
+      let msg = Zotero.getString(
+      'sync.error.invalidDataError',
+      [
+      library.name,
+      Zotero.clientName]) +
 
 
-        "\n\n" +
-        Zotero.getString('sync.error.invalidDataError.otherData');
+      "\n\n" +
+      Zotero.getString('sync.error.invalidDataError.otherData');
+
+      // Show warning for My Library
+      if (library.libraryType == 'user') {
+        e.message = msg;
         e.errorType = 'warning';
         e.dialogButtonText = Zotero.getString('general.checkForUpdates');
         e.dialogButtonCallback = () => {
           Zotero.openCheckForUpdatesWindow();
         };
+        e.dialogButton2Text = Zotero.getString('general.moreInformation');
+        e.dialogButton2Callback = () => {
+          Zotero.launchURL('https://www.zotero.org/support/kb/unknown_data_error');
+        };
       }
+      // Otherwise just show in sync button tooltip
+      else {
+        _addTooltipMessage(msg);
+        e.errorType = 'ignore';
+      }
+    }
   });
 
 
   /**
-       * Set the sync icon and sync error icon across all windows
-       *
-       * @param {Error|Error[]|'animate'} errors - An error, an array of errors, or 'animate' to
-       *                                           spin the icon. An empty array will reset the
-       *                                           icons.
-       */
+   * Set the sync icon and sync error icon across all windows
+   *
+   * @param {Error|Error[]|'animate'} errors - An error, an array of errors, or 'animate' to
+   *                                           spin the icon. An empty array will reset the
+   *                                           icons.
+   */
   this.updateIcons = function (errors) {
     if (typeof errors == 'string') {
       var state = errors;
@@ -1299,6 +1380,7 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       if (!Array.isArray(errors)) {
         errors = [errors];
       }
+      errors = errors.filter((o) => o.errorType !== 'ignore');
       var state = this.getPrimaryErrorType(errors);
     }
 
@@ -1325,10 +1407,10 @@ Zotero.Sync.Runner_Module = function (options = {}) {
     }
     // Update all windows
     else {
-        var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
-        getService(Components.interfaces.nsIWindowMediator);
-        var enumerator = wm.getEnumerator('navigator:browser');
-      }
+      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
+      getService(Components.interfaces.nsIWindowMediator);
+      var enumerator = wm.getEnumerator('navigator:browser');
+    }
 
     while (enumerator.hasMoreElements()) {
       var win = enumerator.getNext();
@@ -1361,8 +1443,8 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Set the sync icon tooltip message
-      */
+   * Set the sync icon tooltip message
+   */
   this.setSyncStatus = function (msg) {
     _lastSyncStatus = msg;
 
@@ -1391,9 +1473,9 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Set the state of the sync error icon and add an onclick to populate
-      * the error panel
-      */
+   * Set the state of the sync error icon and add an onclick to populate
+   * the error panel
+   */
   this.updateErrorIcon = function (icon, state, errors) {
     if (!errors || !errors.length) {
       icon.hidden = true;
@@ -1454,11 +1536,11 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       }
       // For unexpected ones, just show a generic message
       else if (e instanceof Zotero.HTTP.UnexpectedStatusException && e.xmlhttp.responseText) {
-          msg = Zotero.Utilities.ellipsize(e.xmlhttp.responseText, 1000, true);
-        } else
-        {
-          msg = e.message;
-        }
+        msg = Zotero.Utilities.ellipsize(e.xmlhttp.responseText, 1000, true);
+      } else
+      {
+        msg = e.message;
+      }
 
       var desc = doc.createElement('description');
       desc.textContent = msg;
@@ -1467,10 +1549,10 @@ Zotero.Sync.Runner_Module = function (options = {}) {
       content.appendChild(desc);
 
       /*// If not an error and there's no explicit button text, don't show
-                                 // button to report errors
-                                 if (e.errorType != 'error' && e.dialogButtonText === undefined) {
-                                 	e.dialogButtonText = null;
-                                 }*/
+      // button to report errors
+      if (e.errorType != 'error' && e.dialogButtonText === undefined) {
+      	e.dialogButtonText = null;
+      }*/
 
       if (e.dialogButtonText !== null) {
         if (e.dialogButtonText === undefined) {
@@ -1484,13 +1566,27 @@ Zotero.Sync.Runner_Module = function (options = {}) {
           var buttonCallback = e.dialogButtonCallback;
         }
 
-        var button = doc.createElement('button');
+        let button = doc.createElement('button');
         button.setAttribute('label', buttonText);
         button.onclick = function () {
           buttonCallback();
           panel.hidePopup();
         };
         buttons.appendChild(button);
+
+        // Second button
+        if (e.dialogButton2Text) {
+          buttonText = e.dialogButton2Text;
+          buttonCallback = e.dialogButton2Callback;
+
+          let button2 = doc.createElement('button');
+          button2.setAttribute('label', buttonText);
+          button2.onclick = () => {
+            buttonCallback();
+            panel.hidePopup();
+          };
+          buttons.insertBefore(button2, button);
+        }
       }
 
       panel.appendChild(box);
@@ -1502,20 +1598,22 @@ Zotero.Sync.Runner_Module = function (options = {}) {
 
 
   /**
-      * Register labels in sync icon tooltip to receive updates
-      *
-      * If no label passed, unregister current label
-      *
-      * @param {Tooltip} [tooltip]
-      */
+   * Register labels in sync icon tooltip to receive updates
+   *
+   * If no label passed, unregister current label
+   *
+   * @param {Tooltip} [tooltip]
+   */
   this.registerSyncStatus = function (tooltip) {
     if (tooltip) {
       _currentSyncStatusLabel = tooltip.firstChild.nextSibling;
       _currentLastSyncLabel = tooltip.firstChild.nextSibling.nextSibling;
+      _currentTooltipMessages = tooltip.querySelector('.sync-button-tooltip-messages');
     } else
     {
       _currentSyncStatusLabel = null;
       _currentLastSyncLabel = null;
+      _currentTooltipMessages = null;
     }
     if (_currentSyncStatusLabel) {
       _updateSyncStatusLabel();
@@ -1549,6 +1647,11 @@ Zotero.Sync.Runner_Module = function (options = {}) {
   });
 
 
+  function _addTooltipMessage(msg) {
+    _tooltipMessages.push(msg.replace(/\n+/g, ' '));
+  };
+
+
   function _updateSyncStatusLabel() {
     if (_lastSyncStatus) {
       _currentSyncStatusLabel.value = _lastSyncStatus;
@@ -1576,15 +1679,28 @@ Zotero.Sync.Runner_Module = function (options = {}) {
     }
     // Don't show "Not yet synced" if a sync is in progress
     else if (_syncInProgress) {
-        _currentLastSyncLabel.hidden = true;
-        return;
-      } else
-      {
-        var msg = Zotero.getString('sync.status.notYetSynced');
-      }
+      _currentLastSyncLabel.hidden = true;
+      return;
+    } else
+    {
+      var msg = Zotero.getString('sync.status.notYetSynced');
+    }
 
     _currentLastSyncLabel.value = Zotero.getString('sync.status.lastSync') + " " + msg;
     _currentLastSyncLabel.hidden = false;
+
+    if (_tooltipMessages.length) {
+      _currentTooltipMessages.textContent = '';
+      for (let message of _tooltipMessages) {
+        let elem = _currentTooltipMessages.ownerDocument.createElementNS(HTML_NS, 'p');
+        elem.textContent = message;
+        _currentTooltipMessages.appendChild(elem);
+      }
+      _currentTooltipMessages.hidden = false;
+    } else
+    {
+      _currentTooltipMessages.hidden = true;
+    }
   }
 
 
