@@ -36,8 +36,8 @@ init();
 Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 /*
-                                                                       * Core functions
-                                                                       */
+ * Core functions
+ */
 (function () {
   // Privileged (public) methods
   this.getStorageDirectory = getStorageDirectory;
@@ -84,9 +84,9 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   };
 
   /**
-      * @property	{Boolean}	locked		Whether all Zotero panes are locked
-      *										with an overlay
-      */
+   * @property	{Boolean}	locked		Whether all Zotero panes are locked
+   *										with an overlay
+   */
   this.__defineGetter__('locked', function () {return _locked;});
   this.__defineSetter__('locked', function (lock) {
     var wasLocked = _locked;
@@ -103,13 +103,13 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   });
 
   /**
-       * @property {Boolean} crashed - True if the application needs to be restarted
-       */
+   * @property {Boolean} crashed - True if the application needs to be restarted
+   */
   this.crashed = false;
 
   /**
-                         * @property	{Boolean}	closing		True if the application is closing.
-                         */
+   * @property	{Boolean}	closing		True if the application is closing.
+   */
   this.closing = false;
 
   this.unlockDeferred;
@@ -133,19 +133,19 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   var _waitingForDBLock = false;
 
   /**
-                                  * Maintains nsITimers to be used when Zotero.wait() completes (to reduce performance penalty
-                                  * of initializing new objects)
-                                  */
+   * Maintains nsITimers to be used when Zotero.wait() completes (to reduce performance penalty
+   * of initializing new objects)
+   */
   var _waitTimers = [];
 
   /**
-                         * Maintains nsITimerCallbacks to be used when Zotero.wait() completes
-                         */
+   * Maintains nsITimerCallbacks to be used when Zotero.wait() completes
+   */
   var _waitTimerCallbacks = [];
 
   /**
-                                 * Maintains running nsITimers in global scope, so that they don't disappear randomly
-                                 */
+   * Maintains running nsITimers in global scope, so that they don't disappear randomly
+   */
   var _runningTimers = new Map();
 
   var _startupTime = new Date();
@@ -157,10 +157,10 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   var _recentErrors = [];
 
   /**
-                           * Initialize the extension
-                           *
-                           * @return {Promise<Boolean>}
-                           */
+   * Initialize the extension
+   *
+   * @return {Promise<Boolean>}
+   */
   this.init = Zotero.Promise.coroutine(function* (options) {
     if (this.initialized || this.skipLoading) {
       return false;
@@ -172,25 +172,6 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     this.uiReadyDeferred = Zotero.Promise.defer();
     this.uiReadyPromise = this.uiReadyDeferred.promise;
 
-    // Add a function to Zotero.Promise to check whether a value is still defined, and if not
-    // to throw a specific error that's ignored by the unhandled rejection handler in
-    // bluebird.js. This allows for easily cancelling promises when they're no longer
-    // needed, for example after a binding is destroyed.
-    //
-    // Example usage:
-    //
-    // getAsync.tap(() => Zotero.Promise.check(this.mode))
-    //
-    // If the binding is destroyed while getAsync() is being resolved and this.mode no longer
-    // exists, subsequent lines won't be run, and nothing will be logged to the console.
-    this.Promise.check = function (val) {
-      if (!val && val !== 0) {
-        let e = new Error();
-        e.name = "ZoteroPromiseInterrupt";
-        throw e;
-      }
-    };
-
     if (options) {
       let opts = [
       'openPane',
@@ -198,7 +179,7 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       'automatedTest',
       'skipBundledFiles'];
 
-      opts.filter(opt => options[opt]).forEach(opt => this[opt] = true);
+      opts.filter((opt) => options[opt]).forEach((opt) => this[opt] = true);
 
       this.forceDataDir = options.forceDataDir;
     }
@@ -228,7 +209,10 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       var version = yield deferred.promise;
     }
     Zotero.version = version;
-    Zotero.isDevBuild = Zotero.version.includes('beta') || Zotero.version.includes('SOURCE');
+    Zotero.isDevBuild = Zotero.version.includes('beta') ||
+    Zotero.version.includes('dev') ||
+    Zotero.version.includes('SOURCE');
+    Zotero.isSourceBuild = Zotero.version.includes('SOURCE');
 
     // OS platform
     var win = Components.classes["@mozilla.org/appshell/appShellService;1"].
@@ -239,6 +223,7 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     this.isWin = this.platform.substr(0, 3) == "Win";
     this.isLinux = this.platform.substr(0, 5) == "Linux";
     this.oscpu = win.navigator.oscpu;
+    this.isBigSurOrLater = this.isMac && !/Mac OS X 10.([1-9]|1[0-5])/.test(win.navigator.oscpu);
 
     // Browser
     Zotero.browser = "g";
@@ -319,8 +304,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
             }
             // Locate data directory
             else if (index == 2) {
-                Zotero.DataDirectory.choose(true);
-              }
+              Zotero.DataDirectory.choose(true);
+            }
 
           } else
           {
@@ -344,16 +329,16 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
             }
             // Locate data directory
             else if (index == 2) {
-                Zotero.DataDirectory.choose(true);
-              }
+              Zotero.DataDirectory.choose(true);
+            }
           }
         };
         return;
       }
       // DEBUG: handle more startup errors
       else {
-          throw e;
-        }
+        throw e;
+      }
     }
 
     if (!this.forceDataDir) {
@@ -373,6 +358,12 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     // Make sure data directory isn't in Dropbox, etc.
     yield Zotero.DataDirectory.checkForUnsafeLocation(dataDir);
 
+    Services.obs.addObserver({
+      observe: function () {
+        Zotero.Session.save();
+      } },
+    "quit-application-granted", false);
+
     // Register shutdown handler to call Zotero.shutdown()
     var _shutdownObserver = { observe: function () {Zotero.shutdown().done();} };
     Services.obs.addObserver(_shutdownObserver, "quit-application", false);
@@ -391,8 +382,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     try {
       var messages = {};
       Services.console.getMessageArray(messages, {});
-      _startupErrors = Object.keys(messages.value).map(i => messages[i]).
-      filter(msg => _shouldKeepError(msg));
+      _startupErrors = Object.keys(messages.value).map((i) => messages[i]).
+      filter((msg) => _shouldKeepError(msg));
     } catch (e) {
       Zotero.logError(e);
     }
@@ -417,8 +408,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   });
 
   /**
-       * Triggers events when initialization finishes
-       */
+   * Triggers events when initialization finishes
+   */
   this.initComplete = function () {
     if (Zotero.initialized) return;
 
@@ -452,10 +443,10 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Initialization function to be called only if Zotero is in full mode
-      *
-      * @return {Promise:Boolean}
-      */
+   * Initialization function to be called only if Zotero is in full mode
+   *
+   * @return {Promise:Boolean}
+   */
   var _initFull = Zotero.Promise.coroutine(function* () {
     if (!(yield _initDB())) return false;
 
@@ -467,7 +458,7 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     var resetDataDirFile = OS.Path.join(dataDir, 'reset-data-directory');
 
     var result = yield Zotero.Promise.all([OS.File.exists(restoreFile), OS.File.exists(resetDataDirFile)]);
-    if (result.some(r => r)) {
+    if (result.some((r) => r)) {
       [Zotero.restoreFromServer, Zotero.resetDataDir] = result;
       try {
         yield Zotero.DB.closeDatabase();
@@ -497,7 +488,7 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
           [
           'sync.server.username',
           'sync.storage.username'].
-          forEach(p => Zotero.Prefs.clear(p));
+          forEach((p) => Zotero.Prefs.clear(p));
 
           // Clear data directory
           Zotero.debug("Deleting data directory files");
@@ -542,9 +533,9 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     Zotero.HTTP.triggerProxyAuth();
 
     // Add notifier queue callbacks to the DB layer
-    Zotero.DB.addCallback('begin', id => Zotero.Notifier.begin(id));
-    Zotero.DB.addCallback('commit', id => Zotero.Notifier.commit(null, id));
-    Zotero.DB.addCallback('rollback', id => Zotero.Notifier.reset(id));
+    Zotero.DB.addCallback('begin', (id) => Zotero.Notifier.begin(id));
+    Zotero.DB.addCallback('commit', (id) => Zotero.Notifier.commit(null, id));
+    Zotero.DB.addCallback('rollback', (id) => Zotero.Notifier.reset(id));
 
     try {
       // Require >=2.1b3 database to ensure proper locking
@@ -582,14 +573,14 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
           }
           // Select new data directory
           else if (index == 1) {
-              let dir = yield Zotero.DataDirectory.choose(true);
-              if (!dir) {
-                quit = true;
-              }
-            } else
-            {
+            let dir = yield Zotero.DataDirectory.choose(true);
+            if (!dir) {
               quit = true;
             }
+          } else
+          {
+            quit = true;
+          }
 
           if (quit) {
             Services.startup.quit(Components.interfaces.nsIAppStartup.eAttemptQuit);
@@ -654,13 +645,13 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
             }
             // Load More Info page
             else if (index == 2) {
-                let uri = Services.io.newURI(kbURL, null, null);
-                let handler = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].
-                getService(Components.interfaces.nsIExternalProtocolService).
-                getProtocolHandlerInfo('http');
-                handler.preferredAction = Components.interfaces.nsIHandlerInfo.useSystemDefault;
-                handler.launchWithURI(uri, null);
-              }
+              let uri = Services.io.newURI(kbURL, null, null);
+              let handler = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].
+              getService(Components.interfaces.nsIExternalProtocolService).
+              getProtocolHandlerInfo('http');
+              handler.preferredAction = Components.interfaces.nsIHandlerInfo.useSystemDefault;
+              handler.launchWithURI(uri, null);
+            }
           };
           throw e;
         }
@@ -683,6 +674,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       yield Zotero.CharacterSets.init();
       yield Zotero.RelationPredicates.init();
 
+      yield Zotero.Session.init();
+
       Zotero.locked = false;
 
       // Initialize various services
@@ -718,18 +711,17 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       yield Zotero.Groups.init();
       yield Zotero.Relations.init();
       yield Zotero.Retractions.init();
+      yield Zotero.Dictionaries.init();
+      Zotero.Reader.init();
 
       // Migrate fields from Extra that can be moved to item fields after a schema update
-      //
-      // Disabled for now
-      //
-      //yield Zotero.Schema.migrateExtraFields();
+      yield Zotero.Schema.migrateExtraFields();
 
       // Load all library data except for items, which are loaded when libraries are first
       // clicked on or if otherwise necessary
       yield Zotero.Promise.each(
       Zotero.Libraries.getAll(),
-      library => Zotero.Promise.coroutine(function* () {
+      (library) => Zotero.Promise.coroutine(function* () {
         yield Zotero.SyncedSettings.loadAll(library.libraryID);
         if (library.libraryType != 'feed') {
           yield Zotero.Collections.loadAll(library.libraryID);
@@ -753,15 +745,17 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     catch (e) {
       Zotero.logError(e);
       if (!Zotero.startupError) {
-        Zotero.startupError = Zotero.getString('startupError') + "\n\n" + (e.stack || e);
+        Zotero.startupError = Zotero.getString('startupError', Zotero.appName) + "\n\n" +
+        Zotero.getString('db.integrityCheck.reportInForums') + "\n\n" + (
+        e.stack || e);
       }
       return false;
     }
   });
 
   /**
-       * Initializes the DB connection
-       */
+   * Initializes the DB connection
+   */
   var _initDB = Zotero.Promise.coroutine(function* (haveReleasedLock) {
     // Initialize main database connection
     Zotero.DB = new Zotero.DBConnection('zotero');
@@ -783,11 +777,11 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       }
       // Test write access on Zotero database
       else if (!Zotero.File.pathToFile(dbfile).isWritable()) {
-          var msg = 'Cannot write to ' + dbfile;
-        } else
-        {
-          var msg = false;
-        }
+        var msg = 'Cannot write to ' + dbfile;
+      } else
+      {
+        var msg = false;
+      }
 
       if (msg) {
         var e = {
@@ -802,15 +796,17 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       if (_checkDataDirAccessError(e)) {}
       // Storage busy
       else if (e.message.includes('2153971713')) {
-          Zotero.startupError = Zotero.getString('startupError.databaseInUse') + "\n\n" +
-          Zotero.getString(
-          "startupError.close" + (Zotero.isStandalone ? 'Firefox' : 'Standalone'));
+        Zotero.startupError = Zotero.getString('startupError.databaseInUse') + "\n\n" +
+        Zotero.getString(
+        "startupError.close" + (Zotero.isStandalone ? 'Firefox' : 'Standalone'));
 
-        } else
-        {
-          let stack = e.stack ? Zotero.Utilities.Internal.filterStack(e.stack) : null;
-          Zotero.startupError = Zotero.getString('startupError') + "\n\n" + (stack || e);
-        }
+      } else
+      {
+        let stack = e.stack ? Zotero.Utilities.Internal.filterStack(e.stack) : null;
+        Zotero.startupError = Zotero.getString('startupError', Zotero.appName) + "\n\n" +
+        Zotero.getString('db.integrityCheck.reportInForums') + "\n\n" + (
+        stack || e);
+      }
 
       Zotero.debug(e.toString(), 1);
       Components.utils.reportError(e); // DEBUG: doesn't always work
@@ -838,26 +834,26 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     // software preventing Zotero from accessing the selected directory (particularly if it's
     // a Firefox profile)
     else {
-        msg += "\n\n" +
-        Zotero.getString('dataDir.moveToDefaultLocation', Zotero.clientName) +
-        "\n\n" +
-        Zotero.getString(
-        'dataDir.migration.failure.full.current', Zotero.DataDirectory.dir) +
+      msg += "\n\n" +
+      Zotero.getString('dataDir.moveToDefaultLocation', Zotero.clientName) +
+      "\n\n" +
+      Zotero.getString(
+      'dataDir.migration.failure.full.current', Zotero.DataDirectory.dir) +
 
-        "\n" +
-        Zotero.getString(
-        'dataDir.migration.failure.full.recommended', Zotero.DataDirectory.defaultDir);
+      "\n" +
+      Zotero.getString(
+      'dataDir.migration.failure.full.recommended', Zotero.DataDirectory.defaultDir);
 
-      }
+    }
     Zotero.startupError = msg;
     return true;
   }
 
 
   /**
-     * Called when the DB has been released by another Zotero process to perform necessary 
-     * initialization steps
-     */
+   * Called when the DB has been released by another Zotero process to perform necessary 
+   * initialization steps
+   */
   this.onDBLockReleased = function () {
     if (Zotero.isConnector) {
       // if DB lock is released, switch out of connector mode
@@ -898,7 +894,6 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       }
     } catch (e) {
       Zotero.logError(e);
-      throw e;
     }
   });
 
@@ -957,8 +952,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Launch a file, the best way we can
-      */
+   * Launch a file, the best way we can
+   */
   this.launchFile = function (file) {
     file = Zotero.File.pathToFile(file);
     try {
@@ -999,8 +994,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Launch a file with the given application
-      */
+   * Launch a file with the given application
+   */
   this.launchFileWithApplication = function (filePath, applicationPath) {
     Zotero.debug(`Launching ${filePath} with ${applicationPath}`);
 
@@ -1025,8 +1020,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Launch a URL externally, the best way we can
-      */
+   * Launch a URL externally, the best way we can
+   */
   this.launchURL = function (url) {
     if (!Zotero.Utilities.isHTTPURL(url)) {
       if (Zotero.Utilities.isHTTPURL(url, true)) {
@@ -1034,25 +1029,25 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       }
       // Launch non-HTTP URLs
       else {
-          let schemeRE = /^([a-z][a-z0-9+.-]+):/;
-          let matches = url.match(schemeRE);
-          if (!matches) {
-            throw new Error(`Invalid URL '${url}'`);
-          }
-          let scheme = matches[1];
-          if (['javascript', 'data', 'chrome', 'resource'].includes(scheme)) {
-            throw new Error(`Invalid scheme '${scheme}'`);
-          }
-          let svc = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].
-          getService(Components.interfaces.nsIExternalProtocolService);
-          let found = {};
-          let handlerInfo = svc.getProtocolHandlerInfoFromOS(scheme, found);
-          if (!found.value) {
-            throw new Error(`Handler not found for '${scheme}' URLs`);
-          }
-          svc.loadURI(Services.io.newURI(url, null, null));
-          return;
+        let schemeRE = /^([a-z][a-z0-9+.-]+):/;
+        let matches = url.match(schemeRE);
+        if (!matches) {
+          throw new Error(`Invalid URL '${url}'`);
         }
+        let scheme = matches[1];
+        if (['javascript', 'data', 'chrome', 'resource'].includes(scheme)) {
+          throw new Error(`Invalid scheme '${scheme}'`);
+        }
+        let svc = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].
+        getService(Components.interfaces.nsIExternalProtocolService);
+        let found = {};
+        let handlerInfo = svc.getProtocolHandlerInfoFromOS(scheme, found);
+        if (!found.value) {
+          throw new Error(`Handler not found for '${scheme}' URLs`);
+        }
+        svc.loadURI(Services.io.newURI(url, null, null));
+        return;
+      }
     }
 
     try {
@@ -1091,11 +1086,11 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Opens a URL in the basic viewer, and optionally run a callback on load
-      *
-      * @param {String} uri
-      * @param {Function} [onLoad] - Function to run once URI is loaded; passed the loaded document
-      */
+   * Opens a URL in the basic viewer, and optionally run a callback on load
+   *
+   * @param {String} uri
+   * @param {Function} [onLoad] - Function to run once URI is loaded; passed the loaded document
+   */
   this.openInViewer = function (uri, onLoad) {
     var wm = Services.wm;
     var win = wm.getMostRecentWindow("zotero:basicViewer");
@@ -1127,17 +1122,17 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /*
-      * Debug logging function
-      *
-      * Uses prefs e.z.debug.log and e.z.debug.level (restart required)
-      *
-      * @param {} message
-      * @param {Integer} [level=3]
-      * @param {Integer} [maxDepth]
-      * @param {Boolean|Integer} [stack] Whether to display the calling stack.
-      *   If true, stack is displayed starting from the caller. If an integer,
-      *   that many stack levels will be omitted starting from the caller.
-      */
+   * Debug logging function
+   *
+   * Uses prefs e.z.debug.log and e.z.debug.level (restart required)
+   *
+   * @param {} message
+   * @param {Integer} [level=3]
+   * @param {Integer} [maxDepth]
+   * @param {Boolean|Integer} [stack] Whether to display the calling stack.
+   *   If true, stack is displayed starting from the caller. If an integer,
+   *   that many stack levels will be omitted starting from the caller.
+   */
   function debug(message, level, maxDepth, stack) {
     // Account for this alias
     if (stack === true) {
@@ -1151,11 +1146,11 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /*
-     * Log a message to the Mozilla JS error console
-     *
-     * |type| is a string with one of the flag types in nsIScriptError:
-     *    'error', 'warning', 'exception', 'strict'
-     */
+   * Log a message to the Mozilla JS error console
+   *
+   * |type| is a string with one of the flag types in nsIScriptError:
+   *    'error', 'warning', 'exception', 'strict'
+   */
   function log(message, type, sourceName, sourceLine, lineNumber, columnNumber) {
     var scriptError = Components.classes["@mozilla.org/scripterror;1"].
     createInstance(Components.interfaces.nsIScriptError);
@@ -1178,9 +1173,9 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   }
 
   /**
-     * Log a JS error to the Mozilla error console and debug output
-     * @param {Exception} err
-     */
+   * Log a JS error to the Mozilla error console and debug output
+   * @param {Exception} err
+   */
   function logError(err) {
     Zotero.debug(err, 1);
     log(err.message ? err.message : err.toString(), "error",
@@ -1198,12 +1193,12 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Display an alert in a given window
-      *
-      * @param {Window}
-      * @param {String} title
-      * @param {String} msg
-      */
+   * Display an alert in a given window
+   *
+   * @param {Window}
+   * @param {String} title
+   * @param {String} msg
+   */
   this.alert = function (window, title, msg) {
     this.debug(`Alert:\n\n${msg}`);
     var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
@@ -1213,13 +1208,16 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Display an error message saying that an error has occurred and Zotero needs to be restarted.
-      *
-      * If |popup| is TRUE, display in popup progress window; otherwise, display as items pane message
-      */
+   * Display an error message saying that an error has occurred and Zotero needs to be restarted.
+   *
+   * If |popup| is TRUE, display in popup progress window; otherwise, display as items pane message
+   */
   this.crash = function (popup) {
     this.crashed = true;
 
+    // Check the database after restart
+    Zotero.Schema.setIntegrityCheckRequired(true).catch((e) => this.logError(e));
+
     var reportErrorsStr = Zotero.getString('errorReport.reportErrors');
     var reportInstructions = Zotero.getString('errorReport.reportInstructions', reportErrorsStr);
 
@@ -1254,8 +1252,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       }
       // Display as items pane message
       else {
-          win.ZoteroPane.setItemsPaneMessage(msg, true);
-        }
+        win.ZoteroPane.setItemsPaneMessage(msg, true);
+      }
     }
   };
 
@@ -1286,8 +1284,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Get versions, platform, etc.
-      */
+   * Get versions, platform, etc.
+   */
   this.getSystemInfo = Zotero.Promise.coroutine(function* () {
     var info = {
       version: Zotero.version,
@@ -1311,8 +1309,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-       * @return {Promise<String[]>} - Promise for an array of extension names and versions
-       */
+   * @return {Promise<String[]>} - Promise for an array of extension names and versions
+   */
   this.getInstalledExtensions = Zotero.Promise.method(function () {
     var deferred = Zotero.Promise.defer();
     function onHaveInstalledAddons(installed) {
@@ -1399,12 +1397,12 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Emulates the behavior of window.setTimeout
-      *
-      * @param {Function} func			The function to be called
-      * @param {Integer} ms				The number of milliseconds to wait before calling func
-      * @return {Integer} - ID of timer to be passed to clearTimeout()
-      */
+   * Emulates the behavior of window.setTimeout
+   *
+   * @param {Function} func			The function to be called
+   * @param {Integer} ms				The number of milliseconds to wait before calling func
+   * @return {Integer} - ID of timer to be passed to clearTimeout()
+   */
   var _lastTimeoutID = 0;
   this.setTimeout = function (func, ms) {
     var id = ++_lastTimeoutID;
@@ -1433,14 +1431,16 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Show Zotero pane overlay and progress bar in all windows
-      *
-      * @param {String} msg
-      * @param {Boolean} [determinate=false]
-      * @param {Boolean} [modalOnly=false] - Don't use popup if Zotero pane isn't showing
-      * @return	void
-      */
+   * Show Zotero pane overlay and progress bar in all windows
+   *
+   * @param {String} msg
+   * @param {Boolean} [determinate=false]
+   * @param {Boolean} [modalOnly=false] - Don't use popup if Zotero pane isn't showing
+   * @return	void
+   */
   this.showZoteroPaneProgressMeter = function (msg, determinate, icon, modalOnly) {
+    const HTML_NS = "http://www.w3.org/1999/xhtml";
+
     // If msg is undefined, keep any existing message. If false/null/"", clear.
     // The message is also cleared when the meters are hidden.
     _progressMessage = msg = (msg === undefined ? _progressMessage : msg) || "";
@@ -1475,17 +1475,15 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
       let id = 'zotero-pane-progressmeter';
       let progressMeter = doc.getElementById(id);
       if (!progressMeter) {
-        progressMeter = doc.createElement('progressmeter');
+        progressMeter = doc.createElementNS(HTML_NS, 'progress');
         progressMeter.id = id;
       }
-      progressMeter.setAttribute('mode', 'undetermined');
       if (determinate) {
-        progressMeter.mode = 'determined';
-        progressMeter.value = 0;
+        progressMeter.setAttribute('value', 0);
         progressMeter.max = 1000;
       } else
       {
-        progressMeter.mode = 'undetermined';
+        progressMeter.removeAttribute('value');
       }
       container.appendChild(progressMeter);
 
@@ -1500,8 +1498,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * @param	{Number}	percentage		Percentage complete as integer or float
-      */
+   * @param	{Number}	percentage		Percentage complete as integer or float
+   */
   this.updateZoteroPaneProgressMeter = function (percentage) {
     if (percentage !== null) {
       if (percentage < 0 || percentage > 100) {
@@ -1515,13 +1513,13 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
     }
     for (let pm of _progressMeters) {
       if (percentage !== null) {
-        if (pm.mode == 'undetermined') {
+        if (!pm.hasAttribute('value')) {
           pm.max = 1000;
-          pm.mode = 'determined';
         }
-        pm.value = percentage;
-      } else if (pm.mode === 'determined') {
-        pm.mode = 'undetermined';
+        pm.setAttribute('value', percentage);
+      } else
+      if (pm.hasAttribute('value')) {
+        pm.removeAttribute('value');
       }
     }
     _lastPercentage = percentage;
@@ -1529,8 +1527,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Hide Zotero pane overlay in all windows
-      */
+   * Hide Zotero pane overlay in all windows
+   */
   this.hideZoteroPaneOverlays = function () {
     this.locked = false;
 
@@ -1554,8 +1552,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Adds a listener to be called when Zotero shuts down (even if Firefox is not shut down)
-      */
+   * Adds a listener to be called when Zotero shuts down (even if Firefox is not shut down)
+   */
   this.addShutdownListener = function (listener) {
     _shutdownListeners.push(listener);
   };
@@ -1672,8 +1670,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /*
-      * Clear entries that no longer exist from various tables
-      */
+   * Clear entries that no longer exist from various tables
+   */
   this.purgeDataObjects = Zotero.Promise.coroutine(function* () {
     var d = new Date();
 
@@ -1706,8 +1704,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
   /**
-      * Brings Zotero Standalone to the foreground
-      */
+   * Brings Zotero Standalone to the foreground
+   */
   this.activateStandalone = function () {
     var uri = Services.io.newURI('zotero://select', null, null);
     var handler = Components.classes['@mozilla.org/uriloader/external-protocol-service;1'].
@@ -1718,8 +1716,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   };
 
   /**
-      * Determines whether to keep an error message so that it can (potentially) be reported later
-      */
+   * Determines whether to keep an error message so that it can (potentially) be reported later
+   */
   function _shouldKeepError(msg) {
     const skip = ['CSS Parser', 'content javascript'];
 
@@ -1772,8 +1770,8 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   }
 
   /**
-     * Warn if Zotero Standalone is running as root and clobber the cache directory if it is
-     */
+   * Warn if Zotero Standalone is running as root and clobber the cache directory if it is
+   */
   function _checkRoot() {
     var env = Components.classes["@mozilla.org/process/environment;1"].
     getService(Components.interfaces.nsIEnvironment);
@@ -1824,9 +1822,9 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
   }
 
   /**
-     * Observer for console messages
-     * @namespace
-     */
+   * Observer for console messages
+   * @namespace
+   */
   var ConsoleListener = {
     "QueryInterface": XPCOMUtils.generateQI([Components.interfaces.nsIConsoleMessage,
     Components.interfaces.nsISupports]),
@@ -1840,11 +1838,11 @@ Services.scriptloader.loadSubScript("resource://zotero/polyfill.js");
 
 
 /*
-                  * Handles keyboard shortcut initialization from preferences, optionally
-                  * overriding existing global shortcuts
-                  *
-                  * Actions are configured in ZoteroPane.handleKeyPress()
-                  */
+ * Handles keyboard shortcut initialization from preferences, optionally
+ * overriding existing global shortcuts
+ *
+ * Actions are configured in ZoteroPane.handleKeyPress()
+ */
 Zotero.Keys = new function () {
   this.init = init;
   this.windowInit = windowInit;
@@ -1854,8 +1852,8 @@ Zotero.Keys = new function () {
 
 
   /*
-                   * Called by Zotero.init()
-                   */
+   * Called by Zotero.init()
+   */
   function init() {
     var cmds = Zotero.Prefs.rootBranch.getChildList(ZOTERO_CONFIG.PREF_BRANCH + 'keys', {}, {});
 
@@ -1873,8 +1871,8 @@ Zotero.Keys = new function () {
 
 
   /*
-     * Called by ZoteroPane.onLoad()
-     */
+   * Called by ZoteroPane.onLoad()
+   */
   function windowInit(document) {
     var globalKeys = [
     {
@@ -1916,10 +1914,10 @@ Zotero.Keys = new function () {
 
 
 /**
-      * Add X-Zotero-Version header to HTTP requests to zotero.org
-      *
-      * @namespace
-      */
+ * Identify client when connecting to first-party domains
+ *
+ * @namespace
+ */
 Zotero.VersionHeader = {
   init: function () {
     this.register();
@@ -1934,13 +1932,21 @@ Zotero.VersionHeader = {
     try {
       let channel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);
       let domain = channel.URI.host;
-      if (domain.endsWith(ZOTERO_CONFIG.DOMAIN_NAME)) {
+      // Add X-Zotero-Version header to HTTP requests to zotero.org
+      let isPrimaryDomain = domain == ZOTERO_CONFIG.DOMAIN_NAME ||
+      domain.endsWith('.' + ZOTERO_CONFIG.DOMAIN_NAME);
+      if (isPrimaryDomain) {
         channel.setRequestHeader("X-Zotero-Version", Zotero.version, false);
       } else
       {
-        let ua = channel.getRequestHeader('User-Agent');
-        ua = this.update(domain, ua);
-        channel.setRequestHeader('User-Agent', ua, false);
+        // Use "Firefox/[version]" in user agent if not a proxy check or file sync request
+        let s3RE = /(zoteroproxycheck|zoterofilestorage(test)?)\.s3\.(us-east-1\.)?amazonaws\.com/;
+        let isAppNameDomain = s3RE.test(domain);
+        if (!isAppNameDomain) {
+          let ua = channel.getRequestHeader('User-Agent');
+          ua = this.update(ua);
+          channel.setRequestHeader('User-Agent', ua, false);
+        }
       }
     }
     catch (e) {
@@ -1949,14 +1955,13 @@ Zotero.VersionHeader = {
   },
 
   /**
-      * Replace Zotero/[version] with Firefox/[version] in the default user agent
-      *
-      * @param {String} domain
-      * @param {String} ua - User Agent
-      * @param {String} [testAppName] - App name to look for (necessary in tests, which are
-      *     currently run in Firefox)
-      */
-  update: function (domain, ua, testAppName) {
+   * Replace Zotero/[version] with Firefox/[version] in the default user agent
+   *
+   * @param {String} ua - User Agent
+   * @param {String} [testAppName] - App name to look for (necessary in tests, which are
+   *     currently run in Firefox)
+   */
+  update: function (ua, testAppName) {
     var info = Services.appinfo;
     var appName = testAppName || info.name;
 
@@ -1979,7 +1984,6 @@ Zotero.VersionHeader = {
 Zotero.DragDrop = {
   currentEvent: null,
   currentOrientation: 0,
-  currentSourceNode: null,
 
   getDataFromDataTransfer: function (dataTransfer, firstOnly) {
     var dt = dataTransfer;
@@ -1994,12 +1998,12 @@ Zotero.DragDrop = {
 
     if (dt.types.contains('zotero/collection')) {
       dragData.dataType = 'zotero/collection';
-      let ids = dt.getData('zotero/collection').split(",").map(id => parseInt(id));
+      let ids = dt.getData('zotero/collection').split(",").map((id) => parseInt(id));
       dragData.data = ids;
     } else
     if (dt.types.contains('zotero/item')) {
       dragData.dataType = 'zotero/item';
-      let ids = dt.getData('zotero/item').split(",").map(id => parseInt(id));
+      let ids = dt.getData('zotero/item').split(",").map((id) => parseInt(id));
       dragData.data = ids;
     } else
     {
@@ -2038,28 +2042,8 @@ Zotero.DragDrop = {
   },
 
 
-  getDragSource: function (dataTransfer) {
-    if (!dataTransfer) {
-      //Zotero.debug("Drag data not available", 2);
-      return false;
-    }
-
-    // For items, the drag source is the CollectionTreeRow of the parent window
-    // of the source tree
-    if (dataTransfer.types.contains("zotero/item")) {
-      let sourceNode = dataTransfer.mozSourceNode || this.currentSourceNode;
-      if (!sourceNode || sourceNode.tagName != 'treechildren' ||
-      sourceNode.parentElement.id != 'zotero-items-tree') {
-        return false;
-      }
-      var win = sourceNode.ownerDocument.defaultView;
-      if (win.document.documentElement.getAttribute('windowtype') == 'zotero:search') {
-        return win.ZoteroAdvancedSearch.itemsView.collectionTreeRow;
-      }
-      return win.ZoteroPane.collectionsView.selectedTreeRow;
-    }
-
-    return false;
+  getDragSource: function () {
+    return this.currentDragSource;
   },
 
 
@@ -2075,13 +2059,13 @@ Zotero.DragDrop = {
       }
     }
     return false;
-  }
+  } };
 
 
 
-  /**
-     * Functions for creating and destroying hidden browser objects
-     **/ };
+/**
+ * Functions for creating and destroying hidden browser objects
+ **/
 Zotero.Browser = new function () {
   var nBrowsers = 0;
 
@@ -2114,7 +2098,7 @@ Zotero.Browser = new function () {
     hiddenBrowser.docShell.allowJavascript = options.allowJavaScript !== false;
     hiddenBrowser.docShell.allowMetaRedirects = false;
     hiddenBrowser.docShell.allowPlugins = false;
-    Zotero.debug("Created hidden browser (" + nBrowsers++ + ")");
+    Zotero.debug("Created hidden browser (" + ++nBrowsers + ")");
     return hiddenBrowser;
   };
 
@@ -2133,8 +2117,8 @@ Zotero.Browser = new function () {
 
 
 /*
-      * Implements nsIWebProgressListener
-      */
+ * Implements nsIWebProgressListener
+ */
 Zotero.WebProgressFinishListener = function (onFinish) {
   var _request;
   var _finished = false;
@@ -2189,8 +2173,8 @@ Zotero.WebProgressFinishListener = function (onFinish) {
 };
 
 /*
-    * Saves or loads JSON objects.
-    */
+ * Saves or loads JSON objects.
+ */
 Zotero.JSON = new function () {
   this.serialize = function (arg) {
     Zotero.debug("WARNING: Zotero.JSON.serialize() is deprecated; use JSON.stringify()");
